import logging
from openfisca_france.model.base import *


log = logging.getLogger(__name__)


class cotisations_employeur(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisations sociales employeur'
    set_input = set_input_divide_by_period
    definition_period = MONTH
    calculate_output = calculate_output_add

    def formula(individu, period, parameters):
        cotisations_employeur_contributives = individu('cotisations_employeur_contributives', period)
        cotisations_employeur_non_contributives = individu(
            'cotisations_employeur_non_contributives', period)
        cotisations_employeur_main_d_oeuvre = individu('cotisations_employeur_main_d_oeuvre', period)

        return (
            cotisations_employeur_contributives
            + cotisations_employeur_non_contributives
            + cotisations_employeur_main_d_oeuvre
            )


class cotisations_employeur_contributives(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisations sociales employeur contributives'
    set_input = set_input_divide_by_period
    definition_period = MONTH

    def formula(individu, period, parameters):
        ags = individu('ags', period)
        agff_employeur = individu('agff_employeur', period, options = [ADD])
        agirc_arrco_employeur = individu('agirc_arrco_employeur', period)
        agirc_employeur = individu('agirc_employeur', period, options = [ADD])
        agirc_gmp_employeur = individu('agirc_gmp_employeur', period, options = [ADD])
        apec_employeur = individu('apec_employeur', period)
        arrco_employeur = individu('arrco_employeur', period)
        chomage_employeur = individu('chomage_employeur', period)
        contribution_equilibre_general_employeur = individu('contribution_equilibre_general_employeur', period)
        contribution_equilibre_technique_employeur = individu('contribution_equilibre_technique_employeur', period)
        cotisation_exceptionnelle_temporaire_employeur = individu(
            'cotisation_exceptionnelle_temporaire_employeur', period)
        fonds_emploi_hospitalier = individu('fonds_emploi_hospitalier', period, options = [ADD])
        ircantec_employeur = individu('ircantec_employeur', period, options = [ADD])
        pension_employeur = individu('pension_employeur', period, options = [ADD])
        rafp_employeur = individu('rafp_employeur', period, options = [ADD])
        vieillesse_deplafonnee_employeur = individu('vieillesse_deplafonnee_employeur', period, options = [ADD])
        vieillesse_plafonnee_employeur = individu('vieillesse_plafonnee_employeur', period, options = [ADD])

        cotisations = (
            # prive
            ags
            + agff_employeur
            + agirc_employeur
            + agirc_arrco_employeur
            + agirc_gmp_employeur
            + apec_employeur
            + arrco_employeur
            + chomage_employeur
            + contribution_equilibre_general_employeur
            + contribution_equilibre_technique_employeur
            + cotisation_exceptionnelle_temporaire_employeur
            + vieillesse_deplafonnee_employeur
            + vieillesse_plafonnee_employeur
            # public
            + fonds_emploi_hospitalier
            + ircantec_employeur
            + pension_employeur
            + rafp_employeur
            )

        return cotisations


class cotisations_employeur_non_contributives(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisations sociales employeur non-contributives'
    set_input = set_input_divide_by_period
    definition_period = MONTH

    def formula(individu, period, parameters):
        penibilite = individu('penibilite', period, options = [ADD])
        accident_du_travail = individu('accident_du_travail', period, options = [ADD])
        ati_atiacl = individu('ati_atiacl', period, options = [ADD])
        contribution_solidarite_autonomie = individu('contribution_solidarite_autonomie', period)
        famille = individu('famille', period)
        mmid_employeur = individu('mmid_employeur', period, options = [ADD])
        taxe_salaires = individu('taxe_salaires', period, options = [ADD])
        forfait_social = individu('forfait_social', period, options = [ADD])

        cotisations_employeur_non_contributives = (
            ati_atiacl
            + penibilite
            + accident_du_travail
            + contribution_solidarite_autonomie
            + famille
            + mmid_employeur
            + taxe_salaires
            + forfait_social
            )

        return cotisations_employeur_non_contributives


class cotisations_salariales_contributives(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisations sociales salariales contributives'
    set_input = set_input_divide_by_period
    definition_period = MONTH

    def formula(individu, period, parameters):
        agff_salarie = individu('agff_salarie', period)
        agirc_arrco_salarie = individu('agirc_arrco_salarie', period)
        agirc_salarie = individu('agirc_salarie', period)
        agirc_gmp_salarie = individu('agirc_gmp_salarie', period)
        apec_salarie = individu('apec_salarie', period)
        arrco_salarie = individu('arrco_salarie', period)
        chomage_salarie = individu('chomage_salarie', period)
        contribution_equilibre_general_salarie = individu('contribution_equilibre_general_salarie', period)
        contribution_equilibre_technique_salarie = individu('contribution_equilibre_technique_salarie', period)
        cotisation_exceptionnelle_temporaire_salarie = individu('cotisation_exceptionnelle_temporaire_salarie', period)
        ircantec_salarie = individu('ircantec_salarie', period)
        pension_salarie = individu('pension_salarie', period)
        rafp_salarie = individu('rafp_salarie', period)
        vieillesse_deplafonnee_salarie = individu('vieillesse_deplafonnee_salarie', period)
        vieillesse_plafonnee_salarie = individu('vieillesse_plafonnee_salarie', period)

        cotisations_salariales_contributives = (
            # prive
            agff_salarie
            + agirc_arrco_salarie
            + agirc_salarie
            + agirc_gmp_salarie
            + apec_salarie
            + arrco_salarie
            + chomage_salarie
            + contribution_equilibre_general_salarie
            + contribution_equilibre_technique_salarie
            + cotisation_exceptionnelle_temporaire_salarie
            + vieillesse_deplafonnee_salarie
            + vieillesse_plafonnee_salarie
            # public
            + ircantec_salarie
            + pension_salarie
            + rafp_salarie
            )

        return cotisations_salariales_contributives


class cotisations_salariales_non_contributives(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisations sociales salariales non-contributives'
    set_input = set_input_divide_by_period
    definition_period = MONTH

    def formula(individu, period, parameters):
        contribution_exceptionnelle_solidarite = individu('contribution_exceptionnelle_solidarite', period, options = [ADD])
        mmid_salarie = individu('mmid_salarie', period)

        cotisations_salariales_non_contributives = (
            # prive
            mmid_salarie
            # public
            + contribution_exceptionnelle_solidarite
            )

        return cotisations_salariales_non_contributives


class cotisations_salariales(Variable):
    value_type = float
    entity = Individu
    label = 'Cotisations sociales salariales'
    set_input = set_input_divide_by_period
    definition_period = MONTH
    calculate_output = calculate_output_add

    def formula(individu, period, parameters):
        cotisations_salariales_contributives = individu('cotisations_salariales_contributives', period)
        cotisations_salariales_non_contributives = individu(
            'cotisations_salariales_non_contributives', period)
        exoneration_cotisations_salariales_apprenti = individu('exoneration_cotisations_salariales_apprenti', period, options = [ADD])
        exoneration_cotisations_salarie_stagiaire = individu('exoneration_cotisations_salarie_stagiaire', period, options = [ADD])

        return (
            cotisations_salariales_contributives
            + cotisations_salariales_non_contributives
            + exoneration_cotisations_salariales_apprenti
            + exoneration_cotisations_salarie_stagiaire
            )



from numpy import round, floor, datetime64, maximum

from openfisca_france.model.base import *
from openfisca_france.model.prestations.prestations_familiales.base_ressource import nb_enf
from openfisca_core.periods import Instant


# Prestations familiales
class inactif(Variable):
    value_type = bool
    entity = Famille
    label = 'Parent inactif (PAJE-CLCA)'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class partiel1(Variable):
    value_type = bool
    entity = Famille
    label = 'Parent actif à moins de 50% (PAJE-CLCA)'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class partiel2(Variable):
    value_type = bool
    entity = Famille
    label = 'Parent actif entre 50% et 80% (PAJE-CLCA)'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class opt_colca(Variable):
    value_type = bool
    entity = Famille
    label = 'Opte pour le COLCA'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class empl_dir(Variable):
    value_type = bool
    entity = Famille
    label = 'Emploi direct (CLCMG)'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class ass_mat(Variable):
    value_type = bool
    entity = Famille
    label = 'Assistante maternelle (CLCMG)'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class gar_dom(Variable):
    value_type = bool
    entity = Famille
    label = 'Garde à domicile (CLCMG)'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class paje(Variable):
    value_type = float
    entity = Famille
    label = 'PAJE - Ensemble des prestations'
    reference = 'http://www.caf.fr/aides-et-services/s-informer-sur-les-aides/petite-enfance/la-prestation-d-accueil-du-jeune-enfant-paje'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2017_04(famille, period):
        '''
        Prestation d'accueil du jeune enfant
        '''
        paje_base = famille('paje_base', period)
        paje_naissance = famille('paje_naissance', period)
        paje_prepare = famille('paje_prepare', period)
        paje_cmg = famille('paje_cmg', period)

        return paje_base + (paje_naissance + paje_prepare + paje_cmg)

    def formula_2004_01_01(famille, period):
        '''
        Prestation d'accueil du jeune enfant
        '''
        paje_base = famille('paje_base', period)
        paje_naissance = famille('paje_naissance', period)
        paje_clca = famille('paje_clca', period)
        paje_cmg = famille('paje_cmg', period)
        paje_colca = famille('paje_colca', period)

        return paje_base + (paje_naissance + paje_clca + paje_cmg + paje_colca)


class paje_base(Variable):
    calculate_output = calculate_output_add
    value_type = float
    entity = Famille
    label = 'Allocation de base de la PAJE'
    reference = 'http://vosdroits.service-public.fr/particuliers/F2552.xhtml'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2004(famille, period, parameters):
        couple_biactif = famille('biactivite', period)
        parent_isole = not_(famille('en_couple', period))
        nombre_enfants = famille('af_nbenf', period)
        paje = parameters(period).prestations_sociales.prestations_familiales.petite_enfance.paje
        bmaf = parameters(period).prestations_sociales.prestations_familiales.bmaf.bmaf

        # Le montant, précédemment indexé sur la BMAF, est gelé en 2013, et dégelé en 2018
        date_gel_paje = Instant((2013, 4, 1))
        date_degel_paje = Instant((2018, 4, 1))
        periode_de_gel = date_degel_paje > period.start > date_gel_paje
        indice = parameters(date_gel_paje).prestations_sociales.prestations_familiales.bmaf.bmaf if periode_de_gel else bmaf
        # Le taux ne dépend pas de l'année en cours mais de la réforme en vigueur pour l'année de naissance:
        montant_taux_plein_avant_2014 = indice * paje.paje_cm.montant.allocation_base_taux_plein.avant_2014.taux
        montant_taux_plein_2014_2018 = indice * paje.paje_cm.montant.allocation_base_taux_plein.apres_2014.taux
        montant_taux_plein_apres_2018 = indice * paje.paje_cm.montant.allocation_base_taux_plein.apres_2018.taux

        # Avant réforme d'avril 2014 (enfants nés avant avril 2014)
        def plafond_avant_avril_2014():
            plafond_de_base = paje.paje_plaf.ne_adopte_avant_04_2014.plafond_ressources_0_enfant
            maj_plafond_2_premiers_enfants = paje.paje_plaf.ne_adopte_avant_04_2014.majorations_enfants.premier_2eme_enfant * plafond_de_base
            maj_plafond_par_enfant_sup = paje.paje_plaf.ne_adopte_avant_04_2014.majorations_enfants.troisieme_plus_enfant * plafond_de_base
            maj_plafond_seul_biactif = paje.paje_plaf.ne_adopte_avant_04_2014.biactifs_parents_isoles

            plafond = (
                plafond_de_base
                + min_(nombre_enfants, 2) * maj_plafond_2_premiers_enfants
                + max_(nombre_enfants - 2, 0) * maj_plafond_par_enfant_sup
                + (couple_biactif + parent_isole) * maj_plafond_seul_biactif
                )
            return plafond

        # A partir de la réforme de 2014 et jusqu'à la réforme de 2018 (enfants nés entre le 1er avril 2014 et le 1er avril 2018)
        def plafond_taux_plein_2014_2018():
            plafond_de_base = paje.paje_plaf.ne_adopte_04_2014_et_03_2018.taux_plein.plafond_ressources_0_enfant
            maj_plafond_seul_biactif = paje.paje_plaf.ne_adopte_04_2014_et_03_2018.taux_plein.biactifs_parents_isoles
            maj_plafond_par_enfant = plafond_de_base * paje.paje_plaf.ne_adopte_04_2014_et_03_2018.majorations_enfants.majoration_enfant_supp

            return plafond_apres_ajustement_2014_2018(plafond_de_base, maj_plafond_par_enfant, maj_plafond_seul_biactif)

        def plafond_taux_partiel_2014_2018():
            plafond_de_base = paje.paje_plaf.ne_adopte_04_2014_et_03_2018.taux_partiel.plafond_ressources_0_enfant
            maj_plafond_seul_biactif = paje.paje_plaf.ne_adopte_04_2014_et_03_2018.taux_partiel.biactifs_parents_isoles
            maj_plafond_par_enfant = plafond_de_base * paje.paje_plaf.ne_adopte_04_2014_et_03_2018.majorations_enfants.majoration_enfant_supp

            return plafond_apres_ajustement_2014_2018(plafond_de_base, maj_plafond_par_enfant, maj_plafond_seul_biactif)

        def plafond_apres_ajustement_2014_2018(plafond_de_base, maj_plafond_par_enfant, maj_plafond_seul_biactif):
            plafond = (
                plafond_de_base
                + nombre_enfants * maj_plafond_par_enfant
                + (couple_biactif + parent_isole) * maj_plafond_seul_biactif
                )
            return plafond

        # A partir de la réforme d'avril 2018 (enfants nés apres avril 2018)
        def plafond_taux_plein_apres_2018():
            plafond_de_base = paje.paje_plaf.ne_adopte_apres_04_2018.taux_plein.plafond_ressources_0_enfant
            maj_plafond_2_premiers_enfants = paje.paje_plaf.ne_adopte_apres_04_2018.majorations_enfants.premier_2eme_enfant * plafond_de_base
            maj_plafond_par_enfant_sup = paje.paje_plaf.ne_adopte_apres_04_2018.majorations_enfants.troisieme_plus_enfant * plafond_de_base
            maj_plafond_seul_biactif = paje.paje_plaf.ne_adopte_apres_04_2018.taux_plein.biactifs_parents_isoles

            return plafond_apres_ajustement_apres_2018(plafond_de_base, maj_plafond_2_premiers_enfants, maj_plafond_par_enfant_sup, maj_plafond_seul_biactif)

        def plafond_taux_partiel_apres_2018():
            plafond_de_base = paje.paje_plaf.ne_adopte_apres_04_2018.taux_partiel.plafond_ressources_0_enfant
            maj_plafond_2_premiers_enfants = paje.paje_plaf.ne_adopte_apres_04_2018.majorations_enfants.premier_2eme_enfant * plafond_de_base
            maj_plafond_par_enfant_sup = paje.paje_plaf.ne_adopte_apres_04_2018.majorations_enfants.troisieme_plus_enfant * plafond_de_base
            maj_plafond_seul_biactif = paje.paje_plaf.ne_adopte_apres_04_2018.taux_partiel.biactifs_parents_isoles

            return plafond_apres_ajustement_apres_2018(plafond_de_base, maj_plafond_2_premiers_enfants, maj_plafond_par_enfant_sup, maj_plafond_seul_biactif)

        def plafond_apres_ajustement_apres_2018(plafond_de_base, maj_plafond_2_premiers_enfants, maj_plafond_par_enfant_sup, maj_plafond_seul_biactif):
            plafond = (
                plafond_de_base
                + min_(nombre_enfants, 2) * maj_plafond_2_premiers_enfants
                + max_(nombre_enfants - 2, 0) * maj_plafond_par_enfant_sup
                + (couple_biactif + parent_isole) * maj_plafond_seul_biactif
                )
            return plafond

        a_un_enfant_eligible = famille.any(famille.members('enfant_eligible_paje', period))
        date_plus_jeune = famille.reduce(famille.members('date_naissance', period), maximum, datetime64('1066-01-01'))
        sujet_a_reforme_2014 = date_plus_jeune >= datetime64('2014-04-01')
        sujet_a_reforme_2018 = date_plus_jeune >= datetime64('2018-04-01')
        ne_avant_avril_2014 = True

        plafond_taux_partiel = select(
            [sujet_a_reforme_2018, sujet_a_reforme_2014, ne_avant_avril_2014],
            [plafond_taux_partiel_apres_2018(), plafond_taux_partiel_2014_2018(), plafond_avant_avril_2014()]
            )

        plafond_taux_plein = select(
            [sujet_a_reforme_2018, sujet_a_reforme_2014, ne_avant_avril_2014],
            [plafond_taux_plein_apres_2018(), plafond_taux_plein_2014_2018(), plafond_avant_avril_2014()]
            )

        montant_taux_plein = select(
            [sujet_a_reforme_2018, sujet_a_reforme_2014, ne_avant_avril_2014],
            [montant_taux_plein_apres_2018, montant_taux_plein_2014_2018, montant_taux_plein_avant_2014]
            )

        ressources = famille('prestations_familiales_base_ressources', period)
        montant_taux_partiel = montant_taux_plein / 2

        montant = (
            (ressources <= plafond_taux_plein) * montant_taux_plein
            + (ressources <= plafond_taux_partiel)
            * (ressources > plafond_taux_plein) * montant_taux_partiel
            )

        return a_un_enfant_eligible * montant


class enfant_eligible_paje(Variable):
    value_type = bool
    entity = Individu
    label = 'Enfant ouvrant droit à la PAJE de base'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        age = individu('age', period)
        autonomie_financiere = individu('autonomie_financiere', period)
        age_limite = parameters(period).prestations_sociales.prestations_familiales.petite_enfance.paje.paje_cm.age_limite_enfant_adopte_non

        # L'allocation de base est versée jusqu'au dernier jour du mois civil précédant
        # celui au cours duquel l'enfant atteint l'âge de 3 ans.
        return (age < age_limite) * not_(autonomie_financiere)


class paje_naissance(Variable):
    calculate_output = calculate_output_add
    value_type = float
    entity = Famille
    label = 'Allocation de naissance de la PAJE'
    reference = 'http://vosdroits.service-public.fr/particuliers/F2550.xhtml'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2018_04_01(famille, period, parameters):
        '''
        Prestation d'accueil du jeune enfant - Allocation de naissance
        Références législatives :git
        https://www.legifrance.gouv.fr/affichCodeArticle.do?cidTexte=LEGITEXT000006073189&idArticle=LEGIARTI000006737121&dateTexte=&categorieLien=cid
        '''
        af_nbenf = famille('af_nbenf', period)
        base_ressources = famille('prestations_familiales_base_ressources', period)
        isole = not_(famille('en_couple', period))
        biactivite = famille('biactivite', period)
        paje = parameters(period).prestations_sociales.prestations_familiales.petite_enfance.paje
        bmaf = parameters(period).prestations_sociales.prestations_familiales.bmaf.bmaf
        prime_naissance = round(100 * paje.paje_cm2.montant.prime_naissance * bmaf) / 100

        # Versée au 7ème mois de grossesse
        diff_mois_naissance_periode_i = (famille.members('date_naissance', period).astype('datetime64[M]') - datetime64(period.start, 'M'))
        nb_enfants_eligibles = famille.sum(diff_mois_naissance_periode_i.astype('int') == 2, role = Famille.ENFANT)

        nbenf = af_nbenf + nb_enfants_eligibles  # Ajouter les enfants à naître

        taux_plafond = (
            (nbenf > 0)
            + paje.paje_plaf.ne_adopte_apres_04_2018.majorations_enfants.premier_2eme_enfant * min_(nbenf, 2)
            + paje.paje_plaf.ne_adopte_apres_04_2018.majorations_enfants.troisieme_plus_enfant * max_(nbenf - 2, 0)
            )

        majoration_isole_biactif = isole | biactivite

        plafond_de_ressources = (
            paje.paje_plaf.ne_adopte_apres_04_2018.taux_partiel.plafond_ressources_0_enfant
            * taux_plafond
            + (taux_plafond > 0)
            * paje.paje_plaf.ne_adopte_apres_04_2018.taux_partiel.biactifs_parents_isoles
            * majoration_isole_biactif
            )

        eligible_prime_naissance = (base_ressources <= plafond_de_ressources)

        return prime_naissance * eligible_prime_naissance * nb_enfants_eligibles

    def formula_2014_04_01(famille, period, parameters):
        '''
        Prestation d'accueil du jeune enfant - Allocation de naissance
        Références législatives :git
        https://www.legifrance.gouv.fr/affichCodeArticle.do?cidTexte=LEGITEXT000006073189&idArticle=LEGIARTI000006737121&dateTexte=&categorieLien=cid
        '''
        af_nbenf = famille('af_nbenf', period)
        base_ressources = famille('prestations_familiales_base_ressources', period)
        isole = not_(famille('en_couple', period))
        biactivite = famille('biactivite', period)
        paje = parameters(period).prestations_sociales.prestations_familiales.petite_enfance.paje
        fam_bmaf = parameters(period).prestations_sociales.prestations_familiales.bmaf

        # Le montant de la PAJE est gelé depuis avril 2013.
        date_gel_paje = Instant((2013, 4, 1))
        bmaf = fam_bmaf.bmaf if period.start < date_gel_paje else parameters(date_gel_paje).prestations_sociales.prestations_familiales.bmaf.bmaf
        prime_naissance = round(100 * paje.paje_cm2.montant.prime_naissance * bmaf) / 100

        # Versée au 7ème mois de grossesse
        diff_mois_naissance_periode_i = (famille.members('date_naissance', period).astype('datetime64[M]') - datetime64(period.start, 'M'))
        nb_enfants_eligibles = famille.sum(diff_mois_naissance_periode_i.astype('int') == 2, role = Famille.ENFANT)

        nbenf = af_nbenf + nb_enfants_eligibles  # Ajouter les enfants à naître

        majoration_isole_biactif = isole | biactivite

        plafond_de_ressources = (
            paje.paje_plaf.ne_adopte_04_2014_et_03_2018.taux_partiel.plafond_ressources_0_enfant
            + paje.paje_plaf.ne_adopte_04_2014_et_03_2018.taux_partiel.plafond_ressources_0_enfant * nbenf * paje.paje_plaf.ne_adopte_04_2014_et_03_2018.majorations_enfants.majoration_enfant_supp
            + paje.paje_plaf.ne_adopte_04_2014_et_03_2018.taux_partiel.biactifs_parents_isoles * majoration_isole_biactif
            )

        eligible_prime_naissance = (base_ressources <= plafond_de_ressources)

        return prime_naissance * eligible_prime_naissance * nb_enfants_eligibles

    def formula_2004_01_01(famille, period, parameters):
        '''
        Prestation d'accueil du jeune enfant - Allocation de naissance
        '''
        af_nbenf = famille('af_nbenf', period)
        base_ressources = famille('prestations_familiales_base_ressources', period)
        isole = not_(famille('en_couple', period))
        biactivite = famille('biactivite', period)
        paje = parameters(period).prestations_sociales.prestations_familiales.petite_enfance.paje
        fam_bmaf = parameters(period).prestations_sociales.prestations_familiales.bmaf

        # Le montant de la PAJE est gelé depuis avril 2013.
        date_gel_paje = Instant((2013, 4, 1))
        bmaf = fam_bmaf.bmaf if period.start < date_gel_paje else parameters(date_gel_paje).prestations_sociales.prestations_familiales.bmaf.bmaf
        nais_prime = round(100 * paje.paje_cm2.montant.prime_naissance * bmaf) / 100

        # Versée au 7ème mois de grossesse
        diff_mois_naissance_periode_i = (famille.members('date_naissance', period).astype('datetime64[M]') - datetime64(period.start, 'M'))
        nb_enfants_eligibles = famille.sum(diff_mois_naissance_periode_i.astype('int') == 2, role = Famille.ENFANT)

        nbenf = af_nbenf + nb_enfants_eligibles  # Ajouter le/les enfants à naître

        plaf_tx = (
            (nbenf > 0)
            + paje.paje_plaf.ne_adopte_avant_04_2014.majorations_enfants.premier_2eme_enfant * min_(nbenf, 2)
            + paje.paje_plaf.ne_adopte_avant_04_2014.majorations_enfants.troisieme_plus_enfant * max_(nbenf - 2, 0)
            )

        majo = isole | biactivite

        plaf = (
            paje.paje_plaf.ne_adopte_avant_04_2014.plafond_ressources_0_enfant
            * plaf_tx
            + (plaf_tx > 0)
            * paje.paje_plaf.ne_adopte_avant_04_2014.biactifs_parents_isoles
            * majo
            )

        elig = (base_ressources <= plaf)

        return nais_prime * elig * nb_enfants_eligibles


class paje_prepare(Variable):
    value_type = float
    entity = Famille
    set_input = set_input_divide_by_period
    label = 'Prestation Partagée d’éducation de l’Enfant (PreParE)'
    reference = 'https://www.service-public.fr/particuliers/vosdroits/F32485'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class paje_cmg(Variable):
    calculate_output = calculate_output_add
    value_type = float
    entity = Famille
    label = 'PAJE - Complément de libre choix du mode de garde'
    set_input = set_input_divide_by_period
    reference = [
        'http://www.caf.fr/aides-et-services/s-informer-sur-les-aides/petite-enfance/le-complement-de-libre-choix-du-mode-de-garde',
        'https://www.legifrance.gouv.fr/affichCodeArticle.do;jsessionid=C92307A93BE5F694EB49FE51DC09602C.tplgfr29s_1?idArticle=LEGIARTI000031500755&cidTexte=LEGITEXT000006073189&categorieLien=id&dateTexte='
        ]
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2017_04_01(famille, period, parameters):
        '''
        Prestation d accueil du jeune enfant - Complément de libre choix du mode de garde

        Les conditions

        Vous devez :

            avoir un enfant de moins de 6 ans né, adopté ou recueilli en vue d'adoption à partir du 1er janvier 2004
            employer une assistante maternelle agréée ou une garde à domicile.

        Vous n'avez pas besoin de justifier d'une activité min_ si vous êtes :

            bénéficiaire de l'allocation aux adultes handicapés (Aah)
            au chômage et bénéficiaire de l'allocation d'insertion ou de l'allocation de solidarité spécifique
            bénéficiaire du Revenu de solidarité active (Rsa), sous certaines conditions de ressources étudiées par
            votre Caf, et inscrit dans une démarche d'insertionétudiant (si vous vivez en couple,
            vous devez être tous les deux étudiants).

        Autres conditions à remplir : Assistante maternelle agréée     Garde à domicile
        Son salaire brut ne doit pas dépasser par jour de garde et par enfant 5 fois le montant du Smic horaire brut,
        soit au max 45,00 €.
        Vous ne devez pas bénéficier de l'exonération des cotisations sociales dues pour la personne employée.
        '''
        # Récupération des données

        inactif = famille('inactif', period)
        partiel1 = famille('partiel1', period)
        nombre_enfants = famille('af_nbenf', period)
        base_ressources = famille('prestations_familiales_base_ressources', period.first_month)
        emploi_direct = famille('empl_dir', period)
        assistant_maternel = famille('ass_mat', period)
        garde_a_domicile = famille('gar_dom', period)
        paje_prepare = famille('paje_prepare', period)
        paje = parameters(period).prestations_sociales.prestations_familiales.petite_enfance.paje
        bmaf = parameters(period).prestations_sociales.prestations_familiales.bmaf.bmaf

        aah_i = famille.members('aah', period)
        aah = famille.sum(aah_i)

        etudiant_i = famille.members('etudiant', period)
        parent_etudiant = famille.any(etudiant_i, role = Famille.PARENT)

    # condition de revenu minimal

        cond_age_enf = (nb_enf(famille, period, 0, paje.paje_cmg.limite_age.reduite - 1) > 0)

        # TODO:    cond_rpns    =
        # TODO: RSA insertion, alloc insertion, ass
        cond_nonact = (aah > 0) | parent_etudiant  # | (ass>0)

        cond_eligibilite = cond_age_enf & (not_(inactif) | cond_nonact)

        # Si vous bénéficiez de la PreParE taux plein
        # (= vous ne travaillez plus ou interrompez votre activité professionnelle),
        # vous ne pouvez pas bénéficier du Cmg.
        paje_prepare_inactif = (paje_prepare > 0) * inactif
        eligible = cond_eligibilite * not_(paje_prepare_inactif)

    # Les plafonds de ressource

        seuil_revenus_1 = (
            (nombre_enfants == 1) * paje.plaf_cmg.premier_plafond_ne_adopte_avant_04_2014.enfant
            + (nombre_enfants >= 2) * paje.plaf_cmg.premier_plafond_ne_adopte_avant_04_2014.deux_enfants
            + max_(nombre_enfants - 2, 0) * paje.plaf_cmg.premier_plafond_ne_adopte_avant_04_2014.majoration_enfant_supp
            )

        seuil_revenus_2 = (
            (nombre_enfants == 1) * paje.plaf_cmg.deuxieme_plafond_ne_adopte_avant_04_2014.enfant
            + (nombre_enfants >= 2) * paje.plaf_cmg.deuxieme_plafond_ne_adopte_avant_04_2014.deux_enfants
            + max_(nombre_enfants - 2, 0) * paje.plaf_cmg.deuxieme_plafond_ne_adopte_avant_04_2014.majoration_enfant_supp
            )

    #        Si vous bénéficiez du PreParE taux partiel (= vous travaillez entre 50 et 80% de la durée du travail fixée
    #        dans l'entreprise), vous cumulez intégralement la PreParE et le Cmg.
    #        Si vous bénéficiez du PreParE taux partiel (= vous travaillez à 50% ou moins de la durée
    #        du travail fixée dans l'entreprise), le montant des plafonds Cmg est divisé par 2.

        paje_prepare_temps_partiel = (paje_prepare > 0) * partiel1
        seuil_revenus_1 = seuil_revenus_1 * (1 - .5 * paje_prepare_temps_partiel)
        seuil_revenus_2 = seuil_revenus_2 * (1 - .5 * paje_prepare_temps_partiel)

    # calcul du montant

        montant_cmg = (
            bmaf * (
                1.0 * (nb_enf(famille, period, 0, paje.paje_cmg.limite_age.pleine - 1) > 0)
                + 0.5 * (nb_enf(famille, period, paje.paje_cmg.limite_age.pleine, paje.paje_cmg.limite_age.reduite - 1) > 0)
                ) * (
                    emploi_direct * (
                        (base_ressources < seuil_revenus_1) * paje.paje_cmg.complement_libre_choix_mode_garde.revenus_inferieurs_45_plaf
                        + ((base_ressources >= seuil_revenus_1) & (base_ressources < seuil_revenus_2)) * paje.paje_cmg.complement_libre_choix_mode_garde.revenus_superieurs_45_plaf
                        + (base_ressources >= seuil_revenus_2) * paje.paje_cmg.complement_libre_choix_mode_garde.revenus_superieurs_plaf
                        )
                    + assistant_maternel * (
                        (base_ressources < seuil_revenus_1) * paje.paje_cmg.assistante_mat_asso_entreprise_microcreche.sous_premier_plafond
                        + ((base_ressources >= seuil_revenus_1) & (base_ressources < seuil_revenus_2)) * paje.paje_cmg.assistante_mat_asso_entreprise_microcreche.sous_second_plafond
                        + (base_ressources >= seuil_revenus_2) * paje.paje_cmg.assistante_mat_asso_entreprise_microcreche.apres_second_plafond
                        )
                    + garde_a_domicile * (
                        (base_ressources < seuil_revenus_1) * paje.paje_cmg.garde_domicile.sous_premier_plafond
                        + ((base_ressources >= seuil_revenus_1) & (base_ressources < seuil_revenus_2)) * paje.paje_cmg.garde_domicile.sous_second_plafond
                        + (base_ressources >= seuil_revenus_2) * paje.paje_cmg.garde_domicile.apres_second_plafond)
                    )
            )

        paje_cmg = eligible * montant_cmg
        # TODO: connecter avec le crédit d'impôt
        # TODO vérfiez les règles de cumul
        return paje_cmg

    def formula_2004_01_01(famille, period, parameters):
        '''
        Prestation d accueil du jeune enfant - Complément de libre choix du mode de garde

        Les conditions

        Vous devez :

            avoir un enfant de moins de 6 ans né, adopté ou recueilli en vue d'adoption à partir du 1er janvier 2004
            employer une assistante maternelle agréée ou une garde à domicile.
            avoir une activité professionnelle minimale
                si vous êtes salarié cette activité doit vous procurer un revenu minimum de :
                    si vous vivez seul : une fois la BMAF
                    si vous vivez en couple  soit 2 fois la BMAF
                si vous êtes non salarié, vous devez être à jour de vos cotisations sociales d'assurance vieillesse

        Vous n'avez pas besoin de justifier d'une activité min_ si vous êtes :

            bénéficiaire de l'allocation aux adultes handicapés (Aah)
            au chômage et bénéficiaire de l'allocation d'insertion ou de l'allocation de solidarité spécifique
            bénéficiaire du Revenu de solidarité active (Rsa), sous certaines conditions de ressources étudiées par
            votre Caf, et inscrit dans une démarche d'insertionétudiant (si vous vivez en couple,
            vous devez être tous les deux étudiants).

        Autres conditions à remplir : Assistante maternelle agréée     Garde à domicile
        Son salaire brut ne doit pas dépasser par jour de garde et par enfant 5 fois le montant du Smic horaire brut,
        soit au max 45,00 €.
        Vous ne devez pas bénéficier de l'exonération des cotisations sociales dues pour la personne employée.
        '''
        en_couple = famille('en_couple', period)
        af_nbenf = famille('af_nbenf', period)
        base_ressources = famille('prestations_familiales_base_ressources', period.first_month)
        empl_dir = famille('empl_dir', period)
        ass_mat = famille('ass_mat', period)
        gar_dom = famille('gar_dom', period)
        paje_clca_taux_partiel = famille('paje_clca_taux_partiel', period)
        paje_clca_taux_plein = famille('paje_clca_taux_plein', period)
        paje = parameters(period).prestations_sociales.prestations_familiales.petite_enfance.paje
        P_n_2_af = parameters(period.offset(-2, 'year')).prestations_sociales.prestations_familiales.bmaf
        bmaf = parameters(period).prestations_sociales.prestations_familiales.bmaf.bmaf

        aah_i = famille.members('aah', period)
        aah = famille.sum(aah_i)

        etudiant_i = famille.members('etudiant', period)
        parent_etudiant = famille.any(etudiant_i, role = Famille.PARENT)

        salaire_imposable_i = famille.members('salaire_imposable', period)
        salaire_imposable = famille.sum(salaire_imposable_i, role = Famille.PARENT)

        hsup_i = famille.members('hsup', period)
        hsup = famille.sum(hsup_i, role = Famille.PARENT)

        # condition de revenu minimal

        bmaf_n_2 = P_n_2_af.bmaf
        cond_age_enf = (nb_enf(famille, period, 0, paje.paje_cmg.limite_age.reduite - 1) > 0)
        cond_sal = (salaire_imposable + hsup > 12 * bmaf_n_2 * (1 + en_couple))
    # TODO:    cond_rpns    =
        cond_act = cond_sal  # | cond_rpns

        cond_nonact = (aah > 0) | parent_etudiant  # | (ass>0)
    #  TODO: RSA insertion, alloc insertion, ass
        elig = cond_age_enf & (cond_act | cond_nonact)
        nbenf = af_nbenf

        seuil1 = (
            paje.plaf_cmg.premier_plafond_ne_adopte_avant_04_2014.enfant
            * (nbenf == 1)
            + paje.plaf_cmg.premier_plafond_ne_adopte_avant_04_2014.deux_enfants
            * (nbenf >= 2)
            + max_(nbenf - 2, 0)
            * paje.plaf_cmg.premier_plafond_ne_adopte_avant_04_2014.majoration_enfant_supp
            )

        seuil2 = (
            paje.plaf_cmg.deuxieme_plafond_ne_adopte_avant_04_2014.enfant
            * (nbenf == 1)
            + paje.plaf_cmg.deuxieme_plafond_ne_adopte_avant_04_2014.deux_enfants
            * (nbenf >= 2)
            + max_(nbenf - 2, 0)
            * paje.plaf_cmg.deuxieme_plafond_ne_adopte_avant_04_2014.majoration_enfant_supp
            )

    #        Si vous bénéficiez du Clca taux partiel (= vous travaillez entre 50 et 80% de la durée du travail fixée
    #        dans l'entreprise), vous cumulez intégralement le Clca et le Cmg.
    #        Si vous bénéficiez du Clca taux partiel (= vous travaillez à 50% ou moins de la durée
    #        du travail fixée dans l'entreprise), le montant des plafonds Cmg est divisé par 2.
        seuil1 = seuil1 * (1 - .5 * paje_clca_taux_partiel)
        seuil2 = seuil2 * (1 - .5 * paje_clca_taux_partiel)

        clmg = (
            bmaf
            * (
                1.0 * (nb_enf(famille, period, 0, paje.paje_cmg.limite_age.pleine - 1) > 0)
                + 0.5 * (nb_enf(famille, period, paje.paje_cmg.limite_age.pleine, paje.paje_cmg.limite_age.reduite - 1) > 0)
                )
            * (
                empl_dir * (
                    (base_ressources < seuil1) * paje.paje_cmg.complement_libre_choix_mode_garde.revenus_inferieurs_45_plaf
                    + ((base_ressources >= seuil1) & (base_ressources < seuil2)) * paje.paje_cmg.complement_libre_choix_mode_garde.revenus_superieurs_45_plaf
                    + (base_ressources >= seuil2) * paje.paje_cmg.complement_libre_choix_mode_garde.revenus_superieurs_plaf
                    )
                + ass_mat * (
                    (base_ressources < seuil1) * paje.paje_cmg.assistante_mat_asso_entreprise_microcreche.sous_premier_plafond
                    + ((base_ressources >= seuil1) & (base_ressources < seuil2)) * paje.paje_cmg.assistante_mat_asso_entreprise_microcreche.sous_second_plafond
                    + (base_ressources >= seuil2) * paje.paje_cmg.assistante_mat_asso_entreprise_microcreche.apres_second_plafond
                    )
                + gar_dom * (
                    (base_ressources < seuil1) * paje.paje_cmg.garde_domicile.sous_premier_plafond
                    + ((base_ressources >= seuil1) & (base_ressources < seuil2)) * paje.paje_cmg.garde_domicile.sous_second_plafond
                    + (base_ressources >= seuil2) * paje.paje_cmg.garde_domicile.apres_second_plafond
                    )
                )
            )
        # TODO: connecter avec le crédit d'impôt
        # Si vous bénéficiez du Clca taux plein
        # (= vous ne travaillez plus ou interrompez votre activité professionnelle),
        # vous ne pouvez pas bénéficier du Cmg.
        paje_cmg = elig * not_(paje_clca_taux_plein) * clmg
        # TODO vérfiez les règles de cumul
        return paje_cmg


class ape_avant_cumul(Variable):
    value_type = float
    entity = Famille
    label = "Allocation parentale d'éducation, avant prise en compte de la non-cumulabilité avec le CF et l'APJE"
    end = '2003-12-31'
    reference = "http://fr.wikipedia.org/wiki/Allocation_parentale_d'%C3%A9ducation_en_France"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period, parameters):
        '''
        Allocation parentale d'éducation

        L’allocation parentale d’éducation s’adresse aux parents qui souhaitent arrêter ou
        réduire leur activité pour s’occuper de leurs jeunes enfants, à condition que ceux-ci
        soient nés avant le 01/01/2004. En effet, pour les enfants nés depuis cette date,
        dans le cadre de la Prestation d’Accueil du Jeune Enfant, les parents peuvent bénéficier
        du « complément de libre choix d’activité. »

        Les personnes en couple peuvent toutes deux bénéficier de l’APE à taux plein, mais pas en même temps.
        En revanche, elles peuvent cumuler deux taux partiels, à condition que leur total ne dépasse pas le montant
        du taux plein.

        TODO: cumul,  adoption, triplés,
        Cumul d'allocations : Cette allocation n'est pas cumulable pour un même ménage avec
        - une autre APE (sauf à taux partiel),
        - ou l'allocation pour jeune enfant (APJE) versée à partir de la naissance,
        - ou le complément familial,
        - ou l'allocation d’adulte handicapé (AAH).
        Enfin, il est à noter que cette allocation n’est pas cumulable avec :
        - une pension d’invalidité ou une retraite ;
        - des indemnités journalières de maladie, de maternité ou d’accident du travail ;
        - des allocations chômage. Il est tout de même possible de demander aux ASSEDIC la suspension de ces dernières
          pour percevoir l’APE.

        L'allocation parentale d'éducation n'est pas soumise à condition de ressources, sauf l’APE à taux partiel pour
        les professions non salariées.
        '''
        inactif = famille('inactif', period)
        partiel1 = famille('partiel1', period)
        partiel2 = famille('partiel2', period)
        ape = parameters(period).prestations_sociales.prestations_familiales.education_presence_parentale.ape
        af = parameters(period).prestations_sociales.prestations_familiales.prestations_generales.af

        elig = (nb_enf(famille, period, 0, ape.age_max_enfant - 1) >= 1) & (nb_enf(famille, period, 0, af.af_cm.age2) >= 2)        # Inactif
        # Temps partiel 1
        # Salarié:
        # Temps de travail ne dépassant pas 50 % de la durée du travail fixée dans l'entreprise
        # VRP ou non salarié travaillant à temps partiel:
        # Temps de travail ne dépassant pas 76 heures par mois et un revenu professionnel mensuel inférieur ou égal à
        # (smic_8.27*169*85 %)
        # partiel1 = zeros((12,self.taille))

        # Temps partiel 2
        # Salarié:
        # Salarié: Temps de travail compris entre 50 et 80 % de la durée du travail fixée dans l'entreprise.
        # Temps de travail compris entre 77 et 122 heures par mois et un revenu professionnel mensuel ne dépassant pas
        #  (smic_8.27*169*136 %)
        ape = elig * (inactif * ape.montant_mensuel.taux_inactivite + partiel1 * ape.montant_mensuel.taux_activite_sup_50 + partiel2 * ape.montant_mensuel.taux_activite_sup_80)
        # Cummul APE APJE CF
        return ape  # annualisé


class apje_avant_cumul(Variable):
    value_type = float
    entity = Famille
    label = "Allocation pour le jeune enfant, avant prise en compte de la non-cumulabilité avec le CF et l'APE"
    end = '2003-12-31'
    reference = 'http://vosdroits.service-public.fr/particuliers/F2552.xhtml'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period, parameters):
        '''
        Allocation pour jeune enfant
        '''
        base_ressources = famille('prestations_familiales_base_ressources', period.first_month)
        biactivite = famille('biactivite', period, options = [ADD])
        isole = not_(famille('en_couple', period))

        Papje = parameters(period).prestations_sociales.prestations_familiales.petite_enfance.apje
        Paf = parameters(period).prestations_sociales.prestations_familiales.bmaf
        P_n_2 = parameters(period.start.offset(-2, 'year')).prestations_sociales.prestations_familiales.bmaf

        # TODO: APJE courte voir doc ERF 2006
        nbenf = nb_enf(famille, period, 0, Papje.age_limite_enfants_ouvrant_droit_apje - 1)
        bmaf = Paf.bmaf
        bmaf_n_2 = P_n_2.bmaf
        base = round(Papje.apje_cm.taux_apje * bmaf, 2)
        base2 = round(Papje.apje_cm.taux_apje * bmaf_n_2, 2)
        plaf_tx = (nbenf > 0) + Papje.apje_cm.taux_apje_enfant_1_et_2 * min_(nbenf, 2) + Papje.apje_cm.taux_apje_enfant_3_et_plus * max_(nbenf - 2, 0)
        majo = isole | biactivite
        plaf = Papje.apje_plaf.plaf * plaf_tx + Papje.apje_plaf.majoration_plafond_ressources_0_enfant.plaf_maj * majo
        plaf2 = plaf + 12 * base2

        apje = (nbenf >= 1) * ((base_ressources <= plaf) * base + (base_ressources > plaf) * max_(plaf2 - base_ressources, 0) / 12.0)

        # Pour bénéficier de cette allocation, il faut que tous les enfants du foyer soient nés, adoptés, ou recueillis
        # en vue d’une adoption avant le 1er janvier 2004, et qu’au moins l’un d’entre eux ait moins de 3 ans.
        # Cette allocation est versée du 5ème mois de grossesse jusqu'au mois précédant le 3ème anniversaire de
        # l’enfant.

        # Non cumul APE APJE CF
        #  - L’allocation parentale d’éducation (APE), sauf pour les femmes enceintes.
        #    L’APJE est alors versée du 5ème mois de grossesse jusqu’à la naissance de l’enfant.
        #  - Le CF
        return apje


class ape(Variable):
    value_type = float
    entity = Famille
    label = "Allocation parentale d'éducation"
    end = '2003-12-31'
    reference = "http://fr.wikipedia.org/wiki/Allocation_parentale_d'%C3%A9ducation_en_France"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period):
        '''
        L'allocation de base de la paje n'est pas cumulable avec le complément familial
        '''
        apje_avant_cumul = famille('apje_avant_cumul', period)
        ape_avant_cumul = famille('ape_avant_cumul', period)
        cf_montant = famille('cf_montant', period)

        ape = (apje_avant_cumul < ape_avant_cumul) * (cf_montant < ape_avant_cumul) * ape_avant_cumul
        return round(ape, 2)


class apje(Variable):
    value_type = float
    entity = Famille
    label = 'Allocation pour le jeune enfant'
    end = '2003-12-31'
    reference = 'http://vosdroits.service-public.fr/particuliers/F2552.xhtml'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(famille, period):
        # L'APJE n'est pas cumulable avec le complément familial et l'APE
        apje_avant_cumul = famille('apje_avant_cumul', period)
        ape_avant_cumul = famille('ape_avant_cumul', period)
        cf_montant = famille('cf_montant', period)

        apje = (cf_montant < apje_avant_cumul) * (ape_avant_cumul < apje_avant_cumul) * apje_avant_cumul
        return round(apje, 2)


class paje_clca(Variable):
    calculate_output = calculate_output_add
    value_type = float
    entity = Famille
    label = "PAJE - Complément de libre choix d'activité - remplacée par paje_prepare à partir de 04/2017"
    reference = 'http://vosdroits.service-public.fr/particuliers/F313.xhtml'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    end = '2017-04-01'

    def formula_2004(famille, period, parameters):
        '''
        Prestation d'accueil du jeune enfant - Complément de libre choix d'activité
        'fam'

        Parameters:
        -----------

        age :  âge en mois
        af_nbenf : nombre d'enfants aus sens des allocations familiales
        paje_base : allocation de base de la PAJE
        inactif : indicatrice d'inactivité
        partiel1 : Salarié: Temps de travail ne dépassant pas 50 % de la durée du travail fixée dans l'entreprise pour
                   les salariés VRP ou non salarié travaillant à temps partiel: Temps de travail ne dépassant pas 76
                   heures par mois et un revenu professionnel mensuel inférieur ou égal à (smic_8.27*169*85 %)
        partiel2 :  Salarié: Temps de travail compris entre 50 et 80 % de la durée du travail fixée dans l'entreprise.
                    VRP ou non salarié travaillant à temps partiel: Temps de travail compris entre 77 et 122 heures
                    par mois et un revenu professionnel mensuel ne dépassant pas (smic_8.27*169*136 %)

        http://www.caf.fr/wps/portal/particuliers/catalogue/metropole/paje
        '''
        af_nbenf = famille('af_nbenf', period)
        paje_base = famille('paje_base', period)
        inactif = famille('inactif', period)
        partiel1 = famille('partiel1', period)
        partiel2 = famille('partiel2', period)

        paje = parameters(period).prestations_sociales.prestations_familiales.petite_enfance.paje
        bmaf = parameters(period).prestations_sociales.prestations_familiales.bmaf.bmaf

        paje_non_nul = paje_base >= 0
        # durée de versement :
        # Pour un seul enfant à charge, le CLCA est versé pendant une période de 6 mois (paje.paje_clca.duree1)
        # à partir de la naissance ou de la cessation des IJ maternité et paternité.
        # A partir du 2ème enfant, il est versé jusqu’au mois précédant le 3ème anniversaire
        # de l’enfant.

        # Calcul de l'année et mois de naissance du cadet
        # TODO: ajuster en fonction de la cessation des IJ etc

        age_en_mois_i = famille.members('age_en_mois', period)
        age_m_benjamin = famille.min(age_en_mois_i, role = Famille.ENFANT)

        condition1 = (af_nbenf == 1) * (age_m_benjamin >= 0) * (age_m_benjamin < paje.paje_clca.duree1)
        age_benjamin = floor(age_m_benjamin / 12)
        condition2 = (age_benjamin <= (paje.paje_cm.age_limite_enfant_adopte_non - 1))
        condition = (af_nbenf >= 2) * condition2 + condition1

        paje_clca = (
            (condition * bmaf) * (
                not_(paje_non_nul) * (
                    inactif * paje.paje_clca.non_beneficiaires_enfant_ne_adopte_avant_04_2014.taux_plein
                    + partiel1 * paje.paje_clca.non_beneficiaires_enfant_ne_adopte_avant_04_2014.taux_partiel_50
                    + partiel2 * paje.paje_clca.non_beneficiaires_enfant_ne_adopte_avant_04_2014.taux_partiel_50_80
                    )
                + paje_non_nul * (
                    inactif * paje.paje_clca.beneficiaires_enfant_ne_adopte_apres_04_2014.taux_plein
                    + partiel1 * paje.paje_clca.beneficiaires_enfant_ne_adopte_apres_04_2014.taux_partiel_50
                    + partiel2 * paje.paje_clca.beneficiaires_enfant_ne_adopte_apres_04_2014.taux_partiel_50_80
                    )
                )
            )
        return paje_clca


class paje_clca_taux_plein(Variable):
    value_type = bool
    entity = Famille
    label = 'Indicatrice Clca taux plein'
    reference = 'http://vosdroits.service-public.fr/particuliers/F313.xhtml'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    end = '2017-04-01'

    def formula_2004_01_01(famille, period):
        paje_clca = famille('paje_clca', period)
        inactif = famille('inactif', period)

        return (paje_clca > 0) * inactif


class paje_clca_taux_partiel(Variable):
    value_type = bool
    entity = Famille
    label = 'Indicatrice Clca taux partiel'
    reference = 'http://vosdroits.service-public.fr/particuliers/F313.xhtml'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    end = '2017-04-01'

    def formula_2004_01_01(famille, period):
        paje_clca = famille('paje_clca', period)
        partiel1 = famille('partiel1', period)

        return (paje_clca > 0) * partiel1

    # TODO gérer les cumuls avec autres revenus et colca voir site caf


class paje_colca(Variable):
    calculate_output = calculate_output_add
    value_type = float
    entity = Famille
    label = "PAJE - Complément optionnel de libre choix d'activité"
    set_input = set_input_divide_by_period
    reference = 'http://vosdroits.service-public.fr/particuliers/F15110.xhtml'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    end = '2017-04-01'

    def formula_2004_01_01(famille, period, parameters):
        '''
        Prestation d'accueil du jeune enfant - Complément optionnel de libre choix du mode de garde
        '''
        af_nbenf = famille('af_nbenf', period)
        opt_colca = famille('opt_colca', period)
        paje_base = famille('paje_base', period)

        paje = parameters(period).prestations_sociales.prestations_familiales.petite_enfance.paje
        bmaf = parameters(period).prestations_sociales.prestations_familiales.bmaf.bmaf

        age_en_mois_i = famille.members('age_en_mois', period)
        age_m_benjamin = famille.min(age_en_mois_i, role = Famille.ENFANT)

        condition = (age_m_benjamin < 12 * paje.paje_clca.colca.age) * (age_m_benjamin >= 0)
        nbenf = af_nbenf
        paje_non_nul = (paje_base > 0)

        paje_colca = (
            opt_colca
            * condition
            * (nbenf >= 3)
            * bmaf
            * (paje_non_nul * paje.paje_clca.colca.taux_plein_duree_arret_predeterminee + not_(paje_non_nul) * paje.paje_clca.colca.sansab)
            )

        return paje_colca



from openfisca_france.model.base import *


class indemnites_journalieres_maternite(Variable):
    value_type = float
    entity = Individu
    label = 'Indemnités journalières de maternité'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class indemnites_journalieres_paternite(Variable):
    value_type = float
    entity = Individu
    label = 'Indemnités journalières de paternité'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class indemnites_journalieres_adoption(Variable):
    value_type = float
    entity = Individu
    label = "Indemnités journalières d'adoption"
    definition_period = MONTH
    set_input = set_input_divide_by_period


class indemnites_journalieres_maladie(Variable):
    value_type = float
    entity = Individu
    label = 'Indemnités journalières de maladie'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class indemnites_journalieres_accident_travail(Variable):
    value_type = float
    entity = Individu
    label = "Indemnités journalières d'accident du travail"
    definition_period = MONTH
    set_input = set_input_divide_by_period


class indemnites_journalieres_maladie_professionnelle(Variable):
    value_type = float
    entity = Individu
    label = 'Indemnités journalières de maladie professionnelle'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class indemnites_journalieres(Variable):
    value_type = float
    label = 'Total des indemnités journalières'
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        ressources = [
            'indemnites_journalieres_maternite',
            'indemnites_journalieres_paternite',
            'indemnites_journalieres_adoption',
            'indemnites_journalieres_maladie',
            'indemnites_journalieres_accident_travail',
            'indemnites_journalieres_maladie_professionnelle',
            ]
        total = sum(individu(ressource, period) for ressource in ressources)

        return total


class indemnites_journalieres_imposables(Variable):
    value_type = float
    label = 'Total des indemnités journalières imposables'
    entity = Individu
    reference = 'http://vosdroits.service-public.fr/particuliers/F3152.xhtml'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        indemnites_journalieres = individu('indemnites_journalieres', period)
        indemnites_journalieres_accident_travail = individu('indemnites_journalieres_accident_travail', period)
        indemnites_journalieres_maladie_professionnelle = individu('indemnites_journalieres_accident_travail', period)
        result = indemnites_journalieres - 0.5 * (
            indemnites_journalieres_accident_travail + indemnites_journalieres_maladie_professionnelle
            )

        return result


class indemnites_journalieres_atexa(Variable):
    value_type = float
    entity = Individu
    label = "Indemnités de l'assurance Accident du Travail des Exploitants Agricoles"
    reference = 'https://www.legifrance.gouv.fr/jorf/id/JORFTEXT000000649169'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class date_arret_de_travail(Variable):
    value_type = date
    default_value = date.min
    entity = Individu
    label = 'Date depuis laquelle la personne est en arrêt de travail'
    definition_period = ETERNITY



'''
This script takes in two CircleCI build numbers
(should be the `build_python2` and `build_python3` of the same workflow)
and outputs if their mean runtime is in the same ball park
as the master branch's mean runtime for the same builds.

Usage example:

    python openfisca_france/scripts/performance/measure_circleci_builds_diff.py 1717 1716
'''

import sys
import requests
import json
import logging


logging.basicConfig(level=logging.INFO)

if len(sys.argv) < 2:
    raise AttributeError(
        '''
        This script needs two CircleCI build numbers to work,
        e.g. python openfisca_france/scripts/performance/measure_circleci_builds_diff.py 1717 1716
        '''
        )

python2_build_number = sys.argv[1]
python3_build_number = sys.argv[2]


MAX_RATIO_STANDARD_DEVIATION = 10
MIN_NUMBER_OF_MASTER_BUILDS = 5


def get_master_branch_performance():
    '''
    Accesses the CircleCI API and gets the info for all available master branch builds.
    :return: a Dict with the master branch build time statistics
    '''
    def mean(numbers):
        return sum(numbers, 0.0) / len(numbers)

    def standard_deviation(numbers):
        mean_of_numbers = mean(numbers)
        variance = mean([(x - mean_of_numbers)**2 for x in numbers])
        return variance**0.5

    api_url = 'https://circleci.com/api/v1.1/project/github/openfisca/openfisca-france/tree/master'
    response = requests.get(api_url)
    if response.status_code == 200:
        response_python = json.loads(response.content.decode('utf-8'))
    else:
        response_python = None
        logging.warning('No connection to the API - Aborting')
        exit(0)

    builds = []

    for response in response_python:
        job_name = response['workflows']['job_name']
        job_status = response['status']

        if job_status == 'success' and (job_name == 'build_python2' or job_name == 'build_python3'):
            builds.append(response['build_time_millis'])

    if len(builds) < MIN_NUMBER_OF_MASTER_BUILDS:
        logging.warning('Too few circle builds on master branch : {} builds'.format(len(builds)))
        exit(1)
    else:
        logging.info('The master branch has {} builds available for analysis.'.format(len(builds)))

    results = dict()
    results['mean'] = mean(builds)
    results['standard_deviation'] = standard_deviation(builds)
    ratio_sd = round(results['standard_deviation'] / results['mean'] * 100, 2)
    if ratio_sd > MAX_RATIO_STANDARD_DEVIATION:
        logging.warning('The standard deviation is unsound : {}'.format(ratio_sd))
        exit(1)
    return results


def get_current_build_performance(python2_build_number, python3_build_number):
    '''
    get the build time for the tests
    :param python2_build_number: a circle ci build number
    :param python3_build_number: a circle ci build number
    :return: a number of milliseconds the build took
    '''
    builds = [python2_build_number, python3_build_number]
    current_build_performance = 0
    for build in builds:
        api_url = 'https://circleci.com/api/v1.1/project/github/openfisca/openfisca-france/{}'.format(build)
        response = requests.get(api_url)
        if response.status_code == 200:
            response_python = json.loads(response.content.decode('utf-8'))
        else:
            logging.warning('Cannot reach build - Aborting')
            exit(1)
        current_build_performance += response_python['build_time_millis']

    return current_build_performance / 2


master_branch = get_master_branch_performance()
current_build = get_current_build_performance(python2_build_number, python3_build_number)
current_master_ratio = abs(master_branch['mean'] - current_build) / master_branch['mean'] * 100

if current_build > master_branch['mean'] * (1. + (MAX_RATIO_STANDARD_DEVIATION / 100.)):
    sys.exit('This build makes the test performance more than {}% worst : {}%'.format(MAX_RATIO_STANDARD_DEVIATION, current_master_ratio))

elif current_build > master_branch['mean'] + master_branch['standard_deviation']:
    sys.exit('This build may be making the test performance worst : {}%'.format(current_master_ratio))

elif current_build < master_branch['mean'] + master_branch['standard_deviation']:
    logging.info('Performances are good : {}%'.format(current_master_ratio))
    sys.exit(0)



import datetime

from openfisca_core import periods
from openfisca_core.tools import assert_near
from ..cache import tax_benefit_system

from openfisca_france.scenarios import init_single_entity

from openfisca_france.reforms.landais_piketty_saez import landais_piketty_saez


def test():
    year = 2013
    reform = landais_piketty_saez(tax_benefit_system)
    scenario = init_single_entity(reform.new_scenario(),
        axes = [[
            dict(
                count = 3,
                max = 30000,
                min = 0,
                name = 'salaire_de_base',
                ),
            ]],
        period = periods.period(year),
        parent1 = dict(date_naissance = datetime.date(year - 40, 1, 1)),
        # parent2 = dict(date_naissance = datetime.date(year - 40, 1, 1)),
        # enfants = [
        #     dict(date_naissance = datetime.date(year - 9, 1, 1)),
        #     dict(date_naissance = datetime.date(year - 9, 1, 1)),
        #     ],
        )

#    reference_simulation = scenario.new_simulation(use_baseline = True)
#
    reform_simulation = scenario.new_simulation()
    reform_assiette_csg = reform_simulation.calculate('assiette_csg', period = year)
    reform_impot_revenu_lps = reform_simulation.calculate('impot_revenu_lps', period = year)
    assert_near(
        -reform_impot_revenu_lps,
        ((reform_assiette_csg - 10000) * .25 / 30000 + .25) * reform_assiette_csg,
        absolute_error_margin = 0.01,
        )
    reform_simulation.calculate('revenu_disponible', period = year)



from openfisca_france.model.base import Variable, Individu, MONTH
from numpy import select

from openfisca_france.model.caracteristiques_socio_demographiques.demographie import DomaineSpecialitesFormation


class aide_permis_pro_btp(Variable):
    value_type = float
    entity = Individu
    label = "Montant de l'aide au permis B PRO BTP"
    reference = 'https://www.probtp.com/part/apprenti/aide-permis.html'
    definition_period = MONTH

    def formula(individu, period, parameters):
        params = parameters(period).prestations_sociales.aides_jeunes.aide_permis_pro_btp

        rfr = individu.foyer_fiscal('rfr', period.this_year)
        montant = select(
            [
                rfr <= params.plafonds.premier_echelon,
                rfr <= params.plafonds.deuxieme_echelon,
                rfr <= params.plafonds.troisieme_echelon,
                ],
            [
                params.montant.premier_echelon,
                params.montant.deuxieme_echelon,
                params.montant.troisieme_echelon,
                ],
            default=0
            )

        age = individu('age', period)
        eligibilite_age = (params.age.minimum <= age) * (age <= params.age.maximum)

        alternant = individu('alternant', period)

        est_en_formation_btp = individu('domaine_specialites_formation', period) == DomaineSpecialitesFormation.domaine_23

        return eligibilite_age * alternant * montant * est_en_formation_btp



import os

from openfisca_core.taxbenefitsystems import TaxBenefitSystem

from openfisca_france.entities import entities
from openfisca_france.model.prelevements_obligatoires.prelevements_sociaux.cotisations_sociales import preprocessing
from openfisca_france.conf.cache_blacklist import cache_blacklist as conf_cache_blacklist
from openfisca_france.situation_examples import couple


COUNTRY_DIR = os.path.dirname(os.path.abspath(__file__))


class FranceTaxBenefitSystem(TaxBenefitSystem):
    '''French tax benefit system'''
    CURRENCY = '€'
    DATA_SOURCES_DIR = os.path.join(COUNTRY_DIR, 'data', 'sources')
    preprocess_parameters = staticmethod(preprocessing.preprocess_parameters)

    REFORMS_DIR = os.path.join(COUNTRY_DIR, 'reformes')

    def __init__(self):
        TaxBenefitSystem.__init__(self, entities)

        param_dir = os.path.join(COUNTRY_DIR, 'parameters')
        self.load_parameters(param_dir)

        self.add_variables_from_directory(os.path.join(COUNTRY_DIR, 'model'))
        self.cache_blacklist = conf_cache_blacklist

        self.open_api_config = {
            'variable_example': 'rsa_montant',
            'parameter_example': 'marche_travail.salaire_minimum.smic.smic_b_horaire',
            'simulation_example': couple,
            }

    def prefill_cache(self):
        # Compute one "zone APL" variable, to pre-load CSV of "code INSEE commune" to "Zone APL".
        from .model.prestations import aides_logement
        aides_logement.preload_zone_apl()
        from .model.prelevements_obligatoires.prelevements_sociaux.contributions_sociales import versement_transport
        versement_transport.preload_taux_versement_transport()



from datetime import date
from numpy import (
    select,
    where,
    logical_not as not_,
    logical_or as or_,
    minimum as min_,
    maximum as max_,
    round as round_,
    )
from openfisca_france.model.base import (
    Variable,
    Famille,
    MONTH,
    set_input_dispatch_by_period,
    set_input_divide_by_period,
    )


class css_cmu_forfait_logement_base(Variable):
    value_type = float
    entity = Famille
    label = "Forfait logement applicable en cas de propriété ou d'occupation à titre gratuit"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    documentation = '''
    Calcule le forfait logement en fonction du nombre de personnes dans le "foyer CMU" et d'un jeu de taux.
    '''

    def formula_2019_11_01(famille, period, parameters):
        nbp_foyer = famille('cmu_nbp_foyer', period)
        P = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.cs.css.forfait_logement_sans_al
        law_rmi_rsa = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.rsa

        montant_rsa_socle = law_rmi_rsa.rsa_m.montant_de_base_du_rsa * (
            1
            + law_rmi_rsa.rsa_maj.maj_montant_max.couples_celibataire_avec_enfant * (nbp_foyer >= 2)
            + law_rmi_rsa.rsa_maj.maj_montant_max.couple_1_enfant_ou_2e_enfant * (nbp_foyer >= 3)
            )

        return 12 * montant_rsa_socle * select(
            [nbp_foyer == 1, nbp_foyer == 2, nbp_foyer > 2],
            [P.taux_1p, P.taux_2p, P.taux_3p_plus]
            )

    def formula_2009_06_01(famille, period, parameters):
        nbp_foyer = famille('cmu_nbp_foyer', period)
        P = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.cs.cmu.forfait_logement_sans_al
        law_rmi_rsa = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.rsa

        montant_rsa_socle = law_rmi_rsa.rsa_m.montant_de_base_du_rsa * (
            1
            + law_rmi_rsa.rsa_maj.maj_montant_max.couples_celibataire_avec_enfant * (nbp_foyer >= 2)
            + law_rmi_rsa.rsa_maj.maj_montant_max.couple_1_enfant_ou_2e_enfant * (nbp_foyer >= 3)
            )

        return 12 * montant_rsa_socle * select(
            [nbp_foyer == 1, nbp_foyer == 2, nbp_foyer > 2],
            [P.taux_1p, P.taux_2p, P.taux_3p_plus]
            )

    def formula(famille, period, parameters):
        nbp_foyer = famille('cmu_nbp_foyer', period)
        P = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.cs.cmu.forfait_logement_sans_al
        law_rmi_rsa = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.rmi

        montant_rsa_socle = law_rmi_rsa.rmi_m.rmi * (
            1
            + law_rmi_rsa.rmi_maj.maj_montant_max.couples * (nbp_foyer >= 2)
            + law_rmi_rsa.rmi_maj.maj_montant_max.couple_1_enfant_ou_2e_enfant * (nbp_foyer >= 3)
            )

        return 12 * montant_rsa_socle * select(
            [nbp_foyer == 1, nbp_foyer == 2, nbp_foyer > 2],
            [P.taux_1p, P.taux_2p, P.taux_3p_plus]
            )


class css_cmu_forfait_logement_al(Variable):
    value_type = float
    entity = Famille
    label = "Forfait logement applicable en cas d'aide au logement, pour la métropole"
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2019_10_01(famille, period, parameters):
        nbp_foyer = famille('cmu_nbp_foyer', period)
        aide_logement = famille('aide_logement', period)
        P = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.cs.css.forfait_logement_avec_al
        law_rmi_rsa = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.rsa

        montant_rsa_socle = law_rmi_rsa.rsa_m.montant_de_base_du_rsa * (
            1
            + law_rmi_rsa.rsa_maj.maj_montant_max.couples_celibataire_avec_enfant * (nbp_foyer >= 2)
            + law_rmi_rsa.rsa_maj.maj_montant_max.couple_1_enfant_ou_2e_enfant * (nbp_foyer >= 3)
            )

        forfait_logement = 12 * montant_rsa_socle * select(
            [nbp_foyer == 1, nbp_foyer == 2, nbp_foyer > 2],
            [P.taux_1p, P.taux_2p, P.taux_3p_plus]
            )

        return (aide_logement > 0) * min_(12 * aide_logement, forfait_logement)

    def formula_2009_06_01(famille, period, parameters):
        nbp_foyer = famille('cmu_nbp_foyer', period)
        aide_logement = famille('aide_logement', period)
        P = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.cs.cmu.forfait_logement_avec_al
        law_rmi_rsa = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.rsa

        montant_rsa_socle = law_rmi_rsa.rsa_m.montant_de_base_du_rsa * (
            1
            + law_rmi_rsa.rsa_maj.maj_montant_max.couples_celibataire_avec_enfant * (nbp_foyer >= 2)
            + law_rmi_rsa.rsa_maj.maj_montant_max.couple_1_enfant_ou_2e_enfant * (nbp_foyer >= 3)
            )

        forfait_logement = 12 * montant_rsa_socle * select(
            [nbp_foyer == 1, nbp_foyer == 2, nbp_foyer > 2],
            [P.taux_1p, P.taux_2p, P.taux_3p_plus]
            )

        return (aide_logement > 0) * min_(12 * aide_logement, forfait_logement)

    def formula(famille, period, parameters):
        nbp_foyer = famille('cmu_nbp_foyer', period)
        aide_logement = famille('aide_logement', period)
        P = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.cs.cmu.forfait_logement_avec_al
        law_rmi_rsa = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.rmi

        montant_rsa_socle = law_rmi_rsa.rmi_m.rmi * (
            1
            + law_rmi_rsa.rmi_maj.maj_montant_max.couples * (nbp_foyer >= 2)
            + law_rmi_rsa.rmi_maj.maj_montant_max.couple_1_enfant_ou_2e_enfant * (nbp_foyer >= 3)
            )

        forfait_logement = 12 * montant_rsa_socle * select(
            [nbp_foyer == 1, nbp_foyer == 2, nbp_foyer > 2],
            [P.taux_1p, P.taux_2p, P.taux_3p_plus]
            )

        return (aide_logement > 0) * min_(12 * aide_logement, forfait_logement)


class cmu_nbp_foyer(Variable):
    value_type = int
    is_period_size_independent = True
    entity = Famille
    label = 'Nombre de personnes dans le foyer CMU-C'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(famille, period, parameters):
        nb_parents = famille('nb_parents', period)
        cmu_nb_pac = famille('cmu_nb_pac', period)

        return nb_parents + cmu_nb_pac


class cmu_nb_pac(Variable):
    value_type = int
    is_period_size_independent = True
    entity = Famille
    label = 'Nombre de personnes à charge au titre de la CMU-C'
    definition_period = MONTH

    def formula(famille, period, parameters):
        P = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.cs.cmu
        age = famille.members('age', period)
        return famille.sum((age >= 0) * (age <= P.age_limite_pac), role = Famille.ENFANT)


class cmu_c_plafond(Variable):
    value_type = float
    entity = Famille
    label = "Plafond annuel de ressources pour l'éligibilité à la CMU-C"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = 'https://www.legifrance.gouv.fr/affichCodeArticle.do?cidTexte=LEGITEXT000006073189&idArticle=LEGIARTI000006753234'

    def formula(famille, period, parameters):
        '''
        - Le plafond dépends du nombre de personnes dans le foyer.
        - À un plafond de base pour une personne, on applique pour chaque personne supplémentaire un certain coefficient supplémentaire
                - Un coefficient pour la 2eme personne,
                - Un coefficient pour les 3e et 4e personne,
                - Un coefficient pour toute personne supplémentaire
        - Si un enfant est en garde alternée, on ne prend en compte que la moitié de son coefficient.
        - Pour savoir quel coefficient est attribué à chaque enfant, il faut trier les enfants de chaque famille par age.
        '''

        cmu = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.cs.cmu
        age_i = famille.members('age_en_mois', period)
        is_couple = (famille('nb_parents', period) == 2)
        is_enfant = famille.members.has_role(Famille.ENFANT)
        cmu_eligible_majoration_dom = famille('cmu_eligible_majoration_dom', period)
        coeff_garde_alt_i = where(famille.members('garde_alternee', period), 0.5, 1)

        # 0 pour l'aîné, 1 pour le cadet, etc.
        rang_dans_fratrie = famille.members.get_rank(famille, - age_i, condition = is_enfant)

        # Famille monoparentale

        coeff_enfant_i = select(
            [rang_dans_fratrie == 0, rang_dans_fratrie <= 2, rang_dans_fratrie >= 3],
            [cmu.coeff_p2, cmu.coeff_p3_p4, cmu.coeff_p5_plus]
            ) * coeff_garde_alt_i

        coeff_monoparental = 1 + famille.sum(coeff_enfant_i, role = Famille.ENFANT)

        # Couple

        coeff_enfant_i = select(
            [rang_dans_fratrie <= 1, rang_dans_fratrie >= 2],
            [cmu.coeff_p3_p4, cmu.coeff_p5_plus]
            ) * coeff_garde_alt_i

        coeff_couple = 1 + cmu.coeff_p2 + famille.sum(coeff_enfant_i, role = Famille.ENFANT)

        coefficient_famille = where(is_couple, coeff_couple, coeff_monoparental)
        coefficient_dom = 1 + cmu_eligible_majoration_dom * cmu.majoration_dom

        plafonds = (
            cmu.plafond_base
            * coefficient_dom
            * coefficient_famille
            )

        return round_(plafonds)


class cmu_eligible_majoration_dom(Variable):
    value_type = bool
    entity = Famille
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(famille, period):
        menage = famille.demandeur.menage
        residence_guadeloupe = menage('residence_guadeloupe', period)
        residence_martinique = menage('residence_martinique', period)
        residence_guyane = menage('residence_guyane', period)
        residence_reunion = menage('residence_reunion', period)

        return residence_guadeloupe | residence_martinique | residence_guyane | residence_reunion


class cmu_c(Variable):
    value_type = bool
    label = 'Éligibilité à la CMU-C'
    entity = Famille
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(famille, period):
        cmu_c_plafond = famille('cmu_c_plafond', period)
        css_cmu_base_ressources = famille('css_cmu_base_ressources', period)
        residence_mayotte = famille.demandeur.menage('residence_mayotte', period)
        css_cmu_acs_eligibilite = famille('css_cmu_acs_eligibilite', period)

        if period.start.date >= date(2016, 1, 1):
            eligibilite_rsa = famille('rsa', period) > 0
        else:
            # Avant 2016, seules les bénéficiaires du RSA socle avait le droit d'office à la CMU.
            rsa_socle = famille('rsa_socle', period)
            rsa_socle_majore = famille('rsa_socle_majore', period)
            rsa_forfait_logement = famille('rsa_forfait_logement', period)
            rsa_base_ressources = famille('rsa_base_ressources', period)
            socle = max_(rsa_socle, rsa_socle_majore)
            rsa = famille('rsa', period)
            eligibilite_rsa = (rsa > 0) * (rsa_base_ressources < socle - rsa_forfait_logement)

        eligibilite_basique = css_cmu_base_ressources <= cmu_c_plafond

        return (
            css_cmu_acs_eligibilite
            * not_(residence_mayotte)
            * or_(eligibilite_basique, eligibilite_rsa)
            )



from openfisca_france.model.base import *


class unites_consommation(Variable):
    value_type = float
    entity = Menage
    label = "Unités de consommation du ménage, selon l'échelle de l'INSEE"
    reference = 'https://insee.fr/fr/metadonnees/definition/c1802'
    definition_period = YEAR

    def formula(menage, period, parameters):
        age_individu = menage.members('age', period.first_month)
        uc_individu = 0.5 * (age_individu >= 14) + 0.3 * (age_individu < 14)
        return 0.5 + menage.sum(uc_individu)  # 1 uc pour la personne de référence


class type_menage(Variable):
    value_type = int
    is_period_size_independent = True
    entity = Menage
    label = 'Type de ménage'
    definition_period = YEAR

    def formula(menage, period):
        '''
        Type de menage
        TODO: prendre les enfants du ménage et non ceux de la famille
        Attention : des erreurs peuvent subsister quand ménage et famille ne coincide pas (cas des ménages complexes)
        '''

        af_nbenf = menage.personne_de_reference.famille('af_nbenf', period.first_month)
        isole = not_(menage.personne_de_reference.famille('en_couple', period.first_month))

        return (
            0 * (isole * (af_nbenf == 0))  # Célibataire
            + 1 * (not_(isole) * (af_nbenf == 0))  # Couple sans enfants
            + 2 * (not_(isole) * (af_nbenf == 1))  # Couple un enfant
            + 3 * (not_(isole) * (af_nbenf == 2))  # Couple deux enfants
            + 4 * (not_(isole) * (af_nbenf == 3))  # Couple trois enfants et plus
            + 5 * (isole * (af_nbenf == 1))  # Famille monoparentale un enfant
            + 6 * (isole * (af_nbenf == 2))  # Famille monoparentale deux enfants
            + 7 * (isole * (af_nbenf == 3))
            )  # Famille monoparentale trois enfants et plus


class revenu_disponible(Variable):
    value_type = float
    entity = Menage
    label = 'Revenu disponible du ménage'
    reference = 'http://fr.wikipedia.org/wiki/Revenu_disponible'
    definition_period = YEAR

    def formula(menage, period, parameters):
        pensions_nettes_i = menage.members('pensions_nettes', period)
        revenus_nets_du_capital_i = menage.members('revenus_nets_du_capital', period)
        revenus_nets_du_travail_i = menage.members('revenus_nets_du_travail', period)
        pensions_nettes = menage.sum(pensions_nettes_i)
        revenus_nets_du_capital = menage.sum(revenus_nets_du_capital_i)
        revenus_nets_du_travail = menage.sum(revenus_nets_du_travail_i)

        impots_directs = menage('impots_directs', period)

        # On prend en compte les PPE touchés par un foyer fiscal dont le déclarant principal est dans le ménage
        ppe_i = menage.members.foyer_fiscal('ppe', period)  # PPE du foyer fiscal auquel appartient chaque membre du ménage
        ppe = menage.sum(ppe_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)  # On somme seulement pour les déclarants principaux

        # On prend en compte les prestations sociales touchées par une famille dont le demandeur est dans le ménage
        prestations_sociales_i = menage.members.famille('prestations_sociales', period)  # PF de la famille auquel appartient chaque membre du ménage
        prestations_sociales = menage.sum(prestations_sociales_i, role = Famille.DEMANDEUR)  # On somme seulement pour les demandeurs

        return (
            revenus_nets_du_travail
            + impots_directs
            + pensions_nettes
            + ppe
            + prestations_sociales
            + revenus_nets_du_capital
            )


class niveau_de_vie(Variable):
    value_type = float
    entity = Menage
    label = 'Niveau de vie du ménage'
    definition_period = YEAR

    def formula(menage, period):
        revenu_disponible = menage('revenu_disponible', period)
        uc = menage('unites_consommation', period)
        return revenu_disponible / uc


class revenus_nets_du_travail(Variable):
    value_type = float
    entity = Individu
    label = 'Revenus nets du travail (salariés et non salariés)'
    reference = 'http://fr.wikipedia.org/wiki/Revenu_du_travail'
    definition_period = YEAR

    def formula(individu, period):
        salaire_net = individu('salaire_net', period, options = [ADD])
        revenus_non_salarie_nets = individu('revenus_non_salarie_nets', period)
        return salaire_net + revenus_non_salarie_nets


class pensions_nettes(Variable):
    value_type = float
    entity = Individu
    label = 'Pensions et revenus de remplacement'
    reference = 'http://fr.wikipedia.org/wiki/Rente'
    definition_period = YEAR

    def formula(individu, period):
        chomage_net = individu('chomage_net', period, options = [ADD])
        retraite_nette = individu('retraite_nette', period, options = [ADD])
        pensions_alimentaires_percues = individu('pensions_alimentaires_percues', period, options = [ADD])
        pensions_invalidite = individu('pensions_invalidite', period, options = [ADD])

        # Revenus du foyer fiscal, que l'on projette uniquement sur le 1er déclarant
        foyer_fiscal = individu.foyer_fiscal
        pensions_alimentaires_versees = foyer_fiscal('pensions_alimentaires_versees', period)
        rente_viagere_titre_onereux = foyer_fiscal('rente_viagere_titre_onereux', period, options = [ADD])
        pen_foyer_fiscal = pensions_alimentaires_versees + rente_viagere_titre_onereux
        pen_foyer_fiscal_projetees = pen_foyer_fiscal * (individu.has_role(FoyerFiscal.DECLARANT_PRINCIPAL))

        return (
            chomage_net
            + retraite_nette
            + pensions_alimentaires_percues
            + pensions_invalidite
            + pen_foyer_fiscal_projetees
            )


class plus_values_base_large(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Montant des plus-values utilisé pour le montant total de revenus du capital'
    definition_period = YEAR

    def formula(foyer_fiscal, period):
        '''
        La CSG sur plus-values n'est pas calculée sur toutes les plus-values : cf. docstring de la variable v1_assiette_csg_plus_values
        Donc, il existe certaines plus-values pour lesquelles on calcul l'impôt sur le revenu (imposition au barème ou forfaitaire),
        mais pour lesquelles on n'a pas de prélèvements sociaux
        Cette variable est l'assiette de plus-values pour lesquelles au moins un prélèvement est calculé. On l'utilise dans le
        calcul du revenu disponible, afin de n'oublier aucun revenu. Elle vaut la somme de assiette_csg_plus_values et rfr_plus_values_hors_rni,
        où l'on enlève les cases communes entre ces deux variables, et où l'on ajoute les variables présentes dans 'revenu_categoriel_plus_values', mais pas présente
        dans assiette_csg_plus_values
        Attention, on n'ajoute pas en revanche 3SA car de notre compréhension, il s'agit de plus-values qui avaient bénéficié
        de reports d'imposition, report qui a expiré. Ce qui veut dire que ces revenus n'avaient pas été imposés lors de leur
        réalisation (ils le sont maintenant), mais avaient été comptés dans le RFR. Donc, il s'agit de revenus qui ne font pas parti
        du revenu courant de cette année.
        Attention : pour les variables de 'revenu_categoriel_plus_values' ajoutées, elles peuvent représenter des montants nets, alors qu'il faudrait le brut. Améliorer ce point
        '''

        v1_assiette_csg_plus_values = foyer_fiscal('assiette_csg_plus_values', period)
        v2_rfr_plus_values_hors_rni = foyer_fiscal('rfr_plus_values_hors_rni', period)

        f3vg = foyer_fiscal('f3vg', period)
        f3we = foyer_fiscal('f3we', period)
        f3vz = foyer_fiscal('f3vz', period)
        f3vt = foyer_fiscal('f3vt', period)

        rpns_pvce_i = foyer_fiscal.members('rpns_pvce', period)
        rpns_pvce = foyer_fiscal.sum(rpns_pvce_i)

        intersection_v1_v2 = f3vg + f3we + f3vz + rpns_pvce + f3vt

        return v1_assiette_csg_plus_values + v2_rfr_plus_values_hors_rni - intersection_v1_v2

    def formula_2013_01_01(foyer_fiscal, period):
        '''
        Cf. docstring période précédente
        '''

        v1_assiette_csg_plus_values = foyer_fiscal('assiette_csg_plus_values', period)
        v2_rfr_plus_values_hors_rni = foyer_fiscal('rfr_plus_values_hors_rni', period)

        f3we = foyer_fiscal('f3we', period)
        f3vz = foyer_fiscal('f3vz', period)
        f3vl = foyer_fiscal('f3vl', period)
        f3wb = foyer_fiscal('f3wb', period)
        f3vt = foyer_fiscal('f3vt', period)

        rpns_pvce_i = foyer_fiscal.members('rpns_pvce', period)
        rpns_pvce = foyer_fiscal.sum(rpns_pvce_i)

        intersection_v1_v2 = f3we + f3vz + rpns_pvce + f3vt
        ajouts_de_rev_cat_pv = f3vl + f3wb

        return v1_assiette_csg_plus_values + v2_rfr_plus_values_hors_rni - intersection_v1_v2 + ajouts_de_rev_cat_pv

    def formula_2014_01_01(foyer_fiscal, period):
        '''
        Cf. docstring période précédente
        '''

        v1_assiette_csg_plus_values = foyer_fiscal('assiette_csg_plus_values', period)
        v2_rfr_plus_values_hors_rni = foyer_fiscal('rfr_plus_values_hors_rni', period)

        f3we = foyer_fiscal('f3we', period)
        f3vz = foyer_fiscal('f3vz', period)
        f3wb = foyer_fiscal('f3wb', period)
        f3vt = foyer_fiscal('f3vt', period)

        rpns_pvce_i = foyer_fiscal.members('rpns_pvce', period)
        rpns_pvce = foyer_fiscal.sum(rpns_pvce_i)

        intersection_v1_v2 = f3we + f3vz + rpns_pvce + f3vt
        ajouts_de_rev_cat_pv = f3wb

        return v1_assiette_csg_plus_values + v2_rfr_plus_values_hors_rni - intersection_v1_v2 + ajouts_de_rev_cat_pv

    def formula_2017_01_01(foyer_fiscal, period):
        '''
        Cf. docstring période précédente
        '''

        v1_assiette_csg_plus_values = foyer_fiscal('assiette_csg_plus_values', period)
        v2_rfr_plus_values_hors_rni = foyer_fiscal('rfr_plus_values_hors_rni', period)

        f3we = foyer_fiscal('f3we', period)
        f3vz = foyer_fiscal('f3vz', period)
        f3wb = foyer_fiscal('f3wb', period)
        f3vt = foyer_fiscal('f3vt', period)
        f3pi = foyer_fiscal('f3pi', period)

        rpns_pvce_i = foyer_fiscal.members('rpns_pvce', period)
        rpns_pvce = foyer_fiscal.sum(rpns_pvce_i)

        intersection_v1_v2 = f3we + f3vz + rpns_pvce + f3vt + f3pi
        ajouts_de_rev_cat_pv = f3wb

        return v1_assiette_csg_plus_values + v2_rfr_plus_values_hors_rni - intersection_v1_v2 + ajouts_de_rev_cat_pv

    def formula_2018_01_01(foyer_fiscal, period):
        '''
        Cf. docstring période précédente
        Pour 2018 et 2019, assiette_csg_plus_values est inclus dans rfr_plus_values_hors_rni
        '''
        f3wb = foyer_fiscal('f3wb', period)

        rfr_plus_values_hors_rni = foyer_fiscal('rfr_plus_values_hors_rni', period)
        ajouts_de_rev_cat_pv = f3wb

        return rfr_plus_values_hors_rni + ajouts_de_rev_cat_pv


class revenus_nets_du_capital(Variable):
    value_type = float
    entity = Individu
    label = 'Revenus du capital nets de prélèvements sociaux'
    reference = 'http://fr.wikipedia.org/wiki/Revenu#Revenu_du_Capital'
    definition_period = YEAR

    def formula(individu, period):
        '''
        Attention : les formules des calculs des prélèvements sociaux sur revenus du capital avant 2013 n'ont pas été verifiées et sont susceptibles de contenir des erreurs
        Note : On part de l'assiette CSG sur les revenus du capital (avec base élargie pour les plus-values), à partir de
        laquelle on fait les deux modifications ci-dessous :
            (1) On enlève les rentes viagères à titre onéreux, qui sont dans cette
                assiette CSG, mais sont déjà dans la variable pensions_nettes pour
                le calcul du revenu disponible. De plus, le concept de rente foncière
                retenu dans cette assiette était le montant après abattement, n'ayant
                pas de fondement économique
                Par conséquent, vu qu'on retranche la CSG sur les revenus du capital,
                qui contient dans sa base les rentes viagèes à titre onéreux, cette variable
                peut être négative
            (2) On change de concept de plus-values (cf. docstring de la variable plus_values_base_large pour mieux comprendre)
        Cette variable est définie au niveau individuel : on projette les revenus du foyer fiscal
        sur le déclarant principal
        '''

        foyer_fiscal = individu.foyer_fiscal
        assiette_csg_revenus_capital = foyer_fiscal('assiette_csg_revenus_capital', period)
        assiette_csg_plus_values = foyer_fiscal('assiette_csg_plus_values', period)
        plus_values_base_large = foyer_fiscal('plus_values_base_large', period)
        rente_viagere_titre_onereux_net = foyer_fiscal('rente_viagere_titre_onereux_net', period)

        revenus_du_capital_cap_avant_prelevements_sociaux = (
            assiette_csg_revenus_capital
            - assiette_csg_plus_values
            + plus_values_base_large
            - rente_viagere_titre_onereux_net
            )

        prelevements_sociaux_revenus_capital = foyer_fiscal('prelevements_sociaux_revenus_capital', period)

        revenus_foyer_fiscal = (
            revenus_du_capital_cap_avant_prelevements_sociaux
            + prelevements_sociaux_revenus_capital
            )
        revenus_foyer_fiscal_projetes = revenus_foyer_fiscal * individu.has_role(FoyerFiscal.DECLARANT_PRINCIPAL)

        return revenus_foyer_fiscal_projetes


class revenus_fonciers_bruts_menage(Variable):
    value_type = float
    entity = Menage
    label = 'Revenus fonciers du ménage après déficits mais avant abattements'
    definition_period = YEAR

    def formula_2013_01_01(menage, period):
        '''
        Il s'agit des bénéfices (ou déficits) fonciers de l'année courante,
        sans abattement. Il s'agit d'une notion davantage économique.
        Cette variable n'est relative qu'à l'activité foncière de l'année.
        C'est pourquoi on ne prend pas en compte la case 4BD, qui
        correspond à des déficits des années antérieures non encore imputés
        au titre de l'impôt sur le revenu
        Formule vérifiée pour les feuilles d'impôts à partir des revenus 2013,
        d'où le fait que la formule commence à cette date-là
        '''
        f4ba_i = menage.members.foyer_fiscal('f4ba', period)
        f4be_i = menage.members.foyer_fiscal('f4be', period)
        f4bb_i = menage.members.foyer_fiscal('f4bb', period)
        f4bc_i = menage.members.foyer_fiscal('f4bc', period)

        f4ba = menage.sum(f4ba_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)
        f4be = menage.sum(f4be_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)
        f4bb = menage.sum(f4bb_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)
        f4bc = menage.sum(f4bc_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)

        return f4ba + f4be - f4bb - f4bc


class revenus_travail_super_bruts_menage(Variable):
    value_type = float
    entity = Menage
    label = 'Revenus du travail super bruts du ménage'
    definition_period = YEAR

    def formula(menage, period):
        '''
        Revenus du travail super bruts du ménage :
        avant CSG-CRDS, cotisations salariales et patronales
        '''
        salaire_net_i = menage.members('salaire_net', period, options = [ADD])
        rpns_i = menage.members('rpns_imposables', period)
        benefices_imputes_microsocial_i = menage.members.foyer_fiscal('microentreprise', period, options = [ADD]) * menage.members.has_role(FoyerFiscal.DECLARANT_PRINCIPAL)
        csg_imposable_salaire_i = menage.members('csg_imposable_salaire', period, options = [ADD])
        csg_deductible_salaire_i = menage.members('csg_deductible_salaire', period, options = [ADD])
        csg_deductible_non_salarie_i = menage.members('csg_deductible_non_salarie', period, options = [ADD])
        crds_salaire_i = menage.members('crds_salaire', period, options = [ADD])
        cotisations_employeur_i = menage.members('cotisations_employeur', period, options = [ADD])
        cotisations_salariales_i = menage.members('cotisations_salariales', period, options = [ADD])
        cotisations_non_salarie_i = menage.members('cotisations_non_salarie', period, options = [ADD])

        salaire_net = menage.sum(salaire_net_i)
        rpns = menage.sum(rpns_i)
        benefices_imputes_microsocial = menage.sum(benefices_imputes_microsocial_i)
        csg_imposable_salaire = menage.sum(csg_imposable_salaire_i)
        csg_deductible_salaire = menage.sum(csg_deductible_salaire_i)
        csg_deductible_non_salarie = menage.sum(csg_deductible_non_salarie_i)
        crds_salaire = menage.sum(crds_salaire_i)
        cotisations_employeur = menage.sum(cotisations_employeur_i)
        cotisations_salariales = menage.sum(cotisations_salariales_i)
        cotisations_non_salarie = menage.sum(cotisations_non_salarie_i)

        return (
            salaire_net
            + rpns
            + benefices_imputes_microsocial  # le montant imputé comme bénéfice étant forfaitaire, on met la même valeur que pour les revenus nets (les cotisations sont compdans cotisations_non_salaries)
            - cotisations_employeur  # On veut ajouter le montant de cotisations. Vu que ce montant est négatif, on met un "moins". Idem pour les autres items ci-dessous
            - cotisations_salariales  # On veut ajouter le montant de cotisations. Vu que ce montant est négatif, on met un "moins". Idem pour les autres items ci-dessous
            - cotisations_non_salarie
            - csg_imposable_salaire
            - csg_deductible_salaire
            - csg_deductible_non_salarie
            - crds_salaire
            )


class revenus_remplacement_pensions_bruts_menage(Variable):
    value_type = float
    entity = Menage
    label = 'Revenus de remplacement et pensions bruts du ménage'
    definition_period = YEAR

    def formula(menage, period):
        '''
        Revenus de remplacement et pensions bruts du ménage : avant CSG et CRDS
        '''
        pensions_nettes_i = menage.members('pensions_nettes', period)
        casa_i = menage.members('casa', period, options = [ADD])
        csg_imposable_chomage_i = menage.members('csg_imposable_chomage', period, options = [ADD])
        csg_deductible_chomage_i = menage.members('csg_deductible_chomage', period, options = [ADD])
        csg_imposable_retraite_i = menage.members('csg_imposable_retraite', period, options = [ADD])
        csg_deductible_retraite_i = menage.members('csg_deductible_retraite', period, options = [ADD])
        crds_chomage_i = menage.members('crds_chomage', period, options = [ADD])
        crds_retraite_i = menage.members('crds_retraite', period, options = [ADD])

        pensions_nettes = menage.sum(pensions_nettes_i)
        casa = menage.sum(casa_i)
        csg_imposable_chomage = menage.sum(csg_imposable_chomage_i)
        csg_deductible_chomage = menage.sum(csg_deductible_chomage_i)
        csg_imposable_retraite = menage.sum(csg_imposable_retraite_i)
        csg_deductible_retraite = menage.sum(csg_deductible_retraite_i)
        crds_chomage = menage.sum(crds_chomage_i)
        crds_retraite = menage.sum(crds_retraite_i)

        return (
            + pensions_nettes
            - casa
            - csg_imposable_chomage  # On veut ajouter le montant de cotisations. Vu que ce montant est négatif, on met un "moins". Idem pour les autres items ci-dessous
            - csg_deductible_chomage
            - csg_imposable_retraite
            - csg_deductible_retraite
            - crds_chomage
            - crds_retraite
            )


class revenus_capitaux_mobiliers_plus_values_bruts_menage(Variable):
    value_type = float
    entity = Menage
    label = 'Revenus bruts des capitaux mobiliers et plus-values du ménage'
    definition_period = YEAR

    def formula(menage, period):
        '''
        Revenus bruts des capitaux mobiliers et plus-values du ménage :
        avant tout abattement et prélèvement social
        '''

        revenus_capitaux_prelevement_forfaitaire_unique_ir_i = menage.members.foyer_fiscal('revenus_capitaux_prelevement_forfaitaire_unique_ir', period, options = [ADD])
        revenus_capitaux_prelevement_forfaitaire_unique_ir = menage.sum(revenus_capitaux_prelevement_forfaitaire_unique_ir_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)
        revenus_capitaux_prelevement_bareme_i = menage.members.foyer_fiscal('revenus_capitaux_prelevement_bareme', period, options = [ADD])
        revenus_capitaux_prelevement_bareme = menage.sum(revenus_capitaux_prelevement_bareme_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)
        revenus_capitaux_prelevement_liberatoire_i = menage.members.foyer_fiscal('revenus_capitaux_prelevement_liberatoire', period, options = [ADD])
        revenus_capitaux_prelevement_liberatoire = menage.sum(revenus_capitaux_prelevement_liberatoire_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)

        interets_pel_moins_12_ans_cel_i = menage.members('interets_pel_moins_12_ans_cel', period)
        interets_pel_moins_12_ans_cel = menage.sum(interets_pel_moins_12_ans_cel_i)
        assurance_vie_ps_exoneree_irpp_pl_i = menage.members.foyer_fiscal('assurance_vie_ps_exoneree_irpp_pl', period)
        assurance_vie_ps_exoneree_irpp_pl = menage.sum(assurance_vie_ps_exoneree_irpp_pl_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)

        plus_values_base_large_i = menage.members.foyer_fiscal('plus_values_base_large', period)
        plus_values_base_large = menage.sum(plus_values_base_large_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)

        return (
            + revenus_capitaux_prelevement_forfaitaire_unique_ir
            + revenus_capitaux_prelevement_bareme
            + revenus_capitaux_prelevement_liberatoire
            + interets_pel_moins_12_ans_cel
            + assurance_vie_ps_exoneree_irpp_pl
            + plus_values_base_large
            )


class revenus_super_bruts_menage(Variable):
    value_type = float
    entity = Menage
    label = 'Revenus super bruts du ménage'
    definition_period = YEAR

    def formula(menage, period):

        revenus_travail_super_bruts_menage = menage('revenus_travail_super_bruts_menage', period)
        revenus_remplacement_pensions_bruts_menage = menage('revenus_remplacement_pensions_bruts_menage', period)
        revenus_fonciers_bruts_menage = menage('revenus_fonciers_bruts_menage', period)
        revenus_capitaux_mobiliers_plus_values_bruts_menage = menage('revenus_capitaux_mobiliers_plus_values_bruts_menage', period)

        return (
            revenus_travail_super_bruts_menage
            + revenus_remplacement_pensions_bruts_menage
            + revenus_fonciers_bruts_menage
            + revenus_capitaux_mobiliers_plus_values_bruts_menage
            )


class prelevements_sociaux_menage(Variable):
    value_type = float
    entity = Menage
    label = 'Prélèvements sociaux du ménage (tous revenus, hors prestations)'
    definition_period = YEAR

    def formula(menage, period):
        csg_i = menage.members('csg', period, options = [ADD])
        csg = menage.sum(csg_i)
        crds_hors_presta_i = menage.members('crds_hors_prestations', period, options = [ADD])
        crds_hors_prestations = menage.sum(crds_hors_presta_i)

        prelevements_sociaux_revenus_capital_hors_csg_crds_i = menage.members.foyer_fiscal('prelevements_sociaux_revenus_capital_hors_csg_crds', period, options = [ADD])
        prelevements_sociaux_revenus_capital_hors_csg_crds = menage.sum(prelevements_sociaux_revenus_capital_hors_csg_crds_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)

        return (
            + csg
            + crds_hors_prestations
            + prelevements_sociaux_revenus_capital_hors_csg_crds
            )


class prestations_sociales(Variable):
    value_type = float
    entity = Famille
    label = 'Prestations sociales'
    reference = 'http://fr.wikipedia.org/wiki/Prestation_sociale'
    definition_period = YEAR

    def formula(famille, period):
        '''
        Prestations sociales
        '''
        prestations_familiales = famille('prestations_familiales', period)
        minima_sociaux = famille('minima_sociaux', period)
        aides_logement = famille('aides_logement', period)
        reduction_loyer_solidarite = famille('reduction_loyer_solidarite', period, options = [ADD])
        aide_exceptionnelle_covid = famille('covid_aide_exceptionnelle_famille_montant', period, options =[ADD])
        fse_i = famille.members('covid_aide_exceptionnelle_tpe_montant', period, options = [ADD])
        fse = famille.sum(fse_i)

        return prestations_familiales + minima_sociaux + aides_logement + reduction_loyer_solidarite + aide_exceptionnelle_covid + fse


class prestations_familiales(Variable):
    value_type = float
    entity = Famille
    label = 'Prestations familiales'
    reference = 'http://www.social-sante.gouv.fr/informations-pratiques,89/fiches-pratiques,91/prestations-familiales,1885/les-prestations-familiales,12626.html'
    definition_period = YEAR

    def formula(famille, period):
        af = famille('af', period, options = [ADD])
        cf = famille('cf', period, options = [ADD])
        ars = famille('ars', period)
        aeeh = famille('aeeh', period, options = [ADD])
        aes = famille('aes', period, options = [ADD])
        paje = famille('paje', period, options = [ADD])
        asf = famille('asf', period, options = [ADD])
        crds_pfam = famille('crds_pfam', period)

        return af + cf + ars + aeeh + aes + paje + asf + crds_pfam


class minimum_vieillesse(Variable):
    calculate_output = calculate_output_add
    value_type = float
    entity = Famille
    label = 'Minimum vieillesse (ASI + ASPA)'
    definition_period = YEAR

    def formula(famille, period):
        return famille.sum(famille.members('asi', period, options = [ADD])) + famille('aspa', period, options = [ADD])


class minima_sociaux(Variable):
    value_type = float
    entity = Famille
    label = 'Minima sociaux'
    reference = 'http://fr.wikipedia.org/wiki/Minima_sociaux'
    definition_period = YEAR

    def formula(famille, period, parameters):
        aah_i = famille.members('aah', period, options = [ADD])
        caah_i = famille.members('caah', period, options = [ADD])
        aah = famille.sum(aah_i)
        caah = famille.sum(caah_i)
        aefa = famille('aefa', period)
        api = famille('api', period, options = [ADD])
        ass_i = famille.members('ass', period, options = [ADD])
        ass = famille.sum(ass_i)
        minimum_vieillesse = famille('minimum_vieillesse', period, options = [ADD])
        # Certaines réformes ayant des effets de bords nécessitent que le rsa soit calculé avant la ppa
        rsa = famille('rsa', period, options = [ADD])
        ppa = famille('ppa', period, options = [ADD])
        psa = famille('psa', period, options = [ADD])
        crds_mini = famille('crds_mini', period, options = [ADD])
        garantie_jeunes_i = famille.members('garantie_jeunes', period, options = [ADD])
        garantie_jeunes = famille.sum(garantie_jeunes_i)

        return aah + caah + minimum_vieillesse + rsa + aefa + api + ass + psa + ppa + crds_mini + garantie_jeunes


class aides_logement(Variable):
    value_type = float
    entity = Famille
    label = 'Aides logement nettes'
    reference = 'http://vosdroits.service-public.fr/particuliers/N20360.xhtml'
    definition_period = YEAR

    def formula(famille, period):
        return famille('aide_logement', period, options = [ADD])


class irpp_economique(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Notion économique de l'IRPP"
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        '''
        Cette variable d'IRPP comptabilise dans les montants
        d'imposition les acomptes qui, dans la déclaration fiscale, sont considérés comme des crédits
        d'impôt. Ajouter ces acomptes au montant "administratif" d'impôt correspond donc au "véritable impôt"
        payé en totalité, alors que la variable 'irpp' correspond à une notion administrative.
        Exemple :
        Certains revenus du capital sont soumis à un prélèvement forfaitaire à la source non libératoire,
        faisant office d'acompte. Puis, l'impôt au barème sur ces revenus est calculé, et confronté à l'acompte.
        Cet acompte, est en case 2CK, et considéré comme un crédit d'impôt. Retrancher de l'impôt au barème ce
        crédit permet d'obtenir l'impôt dû suite à la déclaration de revenus, qui correspond à la variable 'irpp'.
        Cette notion est administrative. L'impôt total payé correspond à cette notion administrative, augmentée des acomptes.
        '''
        irpp = foyer_fiscal('irpp', period)
        acomptes_ir = foyer_fiscal('acomptes_ir', period)

        return irpp - acomptes_ir  # Car par convention, irpp est un montant négatif et acomptes_ir un montant positif


class impots_directs(Variable):
    value_type = float
    entity = Menage
    label = 'Impôts directs'
    reference = 'http://fr.wikipedia.org/wiki/Imp%C3%B4t_direct'
    definition_period = YEAR

    def formula(menage, period, parameters):
        '''
        Pour les impôts définis au niveau du foyer fiscal :
        on prend en compte l'impôt des foyers fiscaux dont le déclarant principal est dans le ménage
        '''
        taxe_habitation = menage('taxe_habitation', period)

        irpp_economique_i = menage.members.foyer_fiscal('irpp_economique', period)
        irpp_economique = menage.sum(irpp_economique_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)

        prelevement_forfaitaire_liberatoire_i = menage.members.foyer_fiscal('prelevement_forfaitaire_liberatoire', period)
        prelevement_forfaitaire_liberatoire = menage.sum(prelevement_forfaitaire_liberatoire_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)

        prelevement_forfaitaire_unique_ir_i = menage.members.foyer_fiscal('prelevement_forfaitaire_unique_ir', period)
        prelevement_forfaitaire_unique_ir = menage.sum(prelevement_forfaitaire_unique_ir_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)

        # On comptabilise ir_pv_immo ici directement, et non pas dans la variable 'irpp', car administrativement, cet impôt n'est pas dans l'irpp, et n'est déclaré dans le formulaire 2042C que pour calculer le revenu fiscal de référence. On colle à la définition administrative, afin d'avoir une variable 'irpp' qui soit comparable à l'IR du simulateur en ligne de la DGFiP
        ir_pv_immo_i = menage.members.foyer_fiscal('ir_pv_immo', period)
        ir_pv_immo = menage.sum(ir_pv_immo_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)

        isf_ifi_i = menage.members.foyer_fiscal('isf_ifi', period)
        isf_ifi = menage.sum(isf_ifi_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)

        prelevement_liberatoire_autoentrepreneur_i = menage.members.foyer_fiscal('microsocial', period)
        prelevement_liberatoire_autoentrepreneur = menage.sum(prelevement_liberatoire_autoentrepreneur_i, role = FoyerFiscal.DECLARANT_PRINCIPAL)

        return (
            taxe_habitation
            + irpp_economique
            + prelevement_forfaitaire_liberatoire
            + prelevement_forfaitaire_unique_ir
            + ir_pv_immo
            + isf_ifi
            + prelevement_liberatoire_autoentrepreneur
            )



import os

from ..model.base import *


dir_path = os.path.join(os.path.dirname(__file__), 'parameters')


# TODO: les baisses de charges n'ont pas été codées car annulées (toute ou en partie ?)
# par le Conseil constitutionnel

class plfr2014(Reform):
    name = 'Projet de Loi de Finances Rectificative 2014'

    class reduction_impot_exceptionnelle(Variable):
        definition_period = YEAR

        def formula_2013_01_01(foyer_fiscal, period, parameters):
            janvier = period.first_month

            nb_adult = foyer_fiscal('nb_adult', period)
            nb_parents = foyer_fiscal.declarant_principal.famille('nb_parents', period = janvier)
            rfr = foyer_fiscal('rfr', period)
            params = parameters(period).plfr2014.reduction_impot_exceptionnelle
            plafond = params.seuil * nb_adult + (nb_parents - nb_adult) * 2 * params.majoration_seuil
            montant = params.montant_plafond * nb_adult
            return min_(max_(plafond + montant - rfr, 0), montant)

    class reductions(Variable):
        label = "Somme des réductions d'impôt à intégrer pour l'année 2013"
        definition_period = YEAR

        def formula_2013_01_01(foyer_fiscal, period, parameters):
            accult = foyer_fiscal('accult', period)
            frais_de_comptabilite = foyer_fiscal('frais_de_comptabilite', period)
            cappme = foyer_fiscal('cappme', period)
            creaen = foyer_fiscal('creaen', period)
            accueil_dans_etablissement_personnes_agees = foyer_fiscal('accueil_dans_etablissement_personnes_agees', period)
            defense_forets_contre_incendies = foyer_fiscal('defense_forets_contre_incendies', period)
            dfppce = foyer_fiscal('dfppce', period)
            doment = foyer_fiscal('doment', period)
            domlog = foyer_fiscal('domlog', period)
            duflot = foyer_fiscal('duflot_pinel_denormandie_metropole', period)
            duflot_om = foyer_fiscal('duflot_pinel_denormandie_om', period)
            reduction_enfants_scolarises = foyer_fiscal('reduction_enfants_scolarises', period)
            gardenf = foyer_fiscal('gardenf', period)
            interets_paiements_differes_agriculteurs = foyer_fiscal('interets_paiements_differes_agriculteurs', period)
            investissement_forestier = foyer_fiscal('ri_investissement_forestier', period)
            invlst = foyer_fiscal('invlst', period)
            ip_net = foyer_fiscal('ip_net', period)
            location_meublee = foyer_fiscal('location_meublee', period)
            mecena = foyer_fiscal('mecena', period)
            mohist = foyer_fiscal('mohist', period)
            protection_patrimoine_naturel = foyer_fiscal('protection_patrimoine_naturel', period)
            prestations_compensatoires = foyer_fiscal('prestations_compensatoires', period)
            reduction_impot_exceptionnelle = foyer_fiscal('reduction_impot_exceptionnelle', period)
            interets_emprunt_reprise_societe = foyer_fiscal('interets_emprunt_reprise_societe', period)
            restauration_patrimoine_bati = foyer_fiscal('restauration_patrimoine_bati', period)
            rente_survie = foyer_fiscal('rente_survie', period)
            saldom = foyer_fiscal('ri_saldom', period)
            scelli = foyer_fiscal('scelli', period)
            sofica = foyer_fiscal('sofica', period)
            souscriptions_parts_fcpi_fip = foyer_fiscal('souscriptions_parts_fcpi_fip', period)
            total_reductions = accult + frais_de_comptabilite + cappme + creaen + accueil_dans_etablissement_personnes_agees + defense_forets_contre_incendies + dfppce + doment + domlog +\
                duflot + duflot_om + reduction_enfants_scolarises + gardenf + interets_paiements_differes_agriculteurs + investissement_forestier + invlst + location_meublee + mecena + mohist + protection_patrimoine_naturel +\
                prestations_compensatoires + interets_emprunt_reprise_societe + restauration_patrimoine_bati + rente_survie + saldom + scelli + sofica + souscriptions_parts_fcpi_fip + reduction_impot_exceptionnelle
            return min_(ip_net, total_reductions)

    def apply(self):
        for variable in [self.reduction_impot_exceptionnelle, self.reductions]:
            self.update_variable(variable)
        self.modify_parameters(modifier_function = modify_parameters)


def modify_parameters(parameters):
    file_path = os.path.join(dir_path, 'plfr2014.yaml')
    plfr2014_parameters_subtree = load_parameter_file(name='plfr2014', file_path=file_path)

    file_path = os.path.join(dir_path, 'plfrss2014.yaml')
    plfrss2014_parameters_subtree = load_parameter_file(name='plfrss2014', file_path=file_path)

    parameters.add_child('plfr2014', plfr2014_parameters_subtree)
    parameters.add_child('plfrss2014', plfrss2014_parameters_subtree)
    return parameters



from numpy import fabs, timedelta64

from openfisca_core.periods import Period

from openfisca_france.model.base import (
    Variable, date, Enum, Individu, MONTH, ADD, min_, not_,
    set_input_dispatch_by_period, set_input_divide_by_period,
    )
from openfisca_france.model.caracteristiques_socio_demographiques.logement import TypesLieuResidence
from openfisca_france.model.revenus.activite.salarie import (
    TypesContrat, TypesLieuEmploiFormation, TypesCategoriesDemandeurEmploi
    )


class aide_mobilite_date_demande(Variable):
    value_type = date
    default_value = date(1, 1, 1)
    entity = Individu
    label = "Date de demande d'évaluation à l'éligibilité de l'aide à la mobilité (AMOB) - (date du fait générateur)"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    reference = 'http://www.bo-pole-emploi.org/bulletinsofficiels/deliberation-n-2021-42-du-8-juin-2021-bope-n2021-43.html?type=dossiers/2021/bope-n-2021-043-du-11-juin-2021'


class distance_activite_domicile(Variable):
    entity = Individu
    value_type = float
    reference = 'http://www.bo-pole-emploi.org/bulletinsofficiels/deliberation-n-2021-42-du-8-juin-2021-bope-n2021-43.html?type=dossiers/2021/bope-n-2021-043-du-11-juin-2021'
    label = 'Distance en kilomètres entre le lieu de l’entretien d’embauche, la reprise d’emploi, la formation, la prestation d’accompagnement, '\
            "l’immersion professionnelle (PMSMP), le concours public ou l’examen certifiant et le lieu de résidence du demandeur d'emploi"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class nombre_allers_retours(Variable):
    entity = Individu
    value_type = float
    reference = 'http://www.bo-pole-emploi.org/bulletinsofficiels/deliberation-n-2021-42-du-8-juin-2021-bope-n2021-43.html?type=dossiers/2021/bope-n-2021-043-du-11-juin-2021'
    label = "Nombre d'aller/retour pour le calcul de l'aide à la mobilité de Pôle emploi - AMOB"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class aide_mobilite_duree_trajet(Variable):
    entity = Individu
    value_type = float
    reference = 'http://www.bo-pole-emploi.org/bulletinsofficiels/deliberation-n-2021-42-du-8-juin-2021-bope-n2021-43.html?type=dossiers/2021/bope-n-2021-043-du-11-juin-2021'
    label = 'Durée en minutes entre le lieu de l’entretien d’embauche, la reprise d’emploi, la formation, la prestation d’accompagnement, '\
            "l’immersion professionnelle (PMSMP), le concours public ou l’examen certifiant et le lieu de résidence du demandeur d'emploi"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class nuitees(Variable):
    entity = Individu
    value_type = int
    reference = 'http://www.bo-pole-emploi.org/bulletinsofficiels/deliberation-n-2021-42-du-8-juin-2021-bope-n2021-43.html?type=dossiers/2021/bope-n-2021-043-du-11-juin-2021'
    label = "Nombre de nuitées pour le calcul de l'aide à la mobilité de Pôle emploi - AMOB"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class repas(Variable):
    entity = Individu
    value_type = int
    reference = 'http://www.bo-pole-emploi.org/bulletinsofficiels/deliberation-n-2021-42-du-8-juin-2021-bope-n2021-43.html?type=dossiers/2021/bope-n-2021-043-du-11-juin-2021'
    label = "Nombre de repas pour le calcul de l'aide à la mobilité de Pôle emploi - AMOB"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class ContexteActivitePoleEmploi(Enum):
    __order__ = 'indetermine formation reprise_emploi recherche_emploi'  # Needed to preserve the enum order in Python 2
    indetermine = 'Indeterminé'
    formation = 'En formation'
    reprise_emploi = "En reprise d'emploi"
    recherche_emploi = "En recherche d'emploi"


class contexte_activite_pole_emploi(Variable):
    value_type = Enum
    possible_values = ContexteActivitePoleEmploi
    default_value = ContexteActivitePoleEmploi.indetermine
    entity = Individu
    label = "Le contexte d'activité au sens de Pôle Emploi"
    definition_period = MONTH


class DispositifsDeFormation(Enum):
    __order__ = 'autre bilan_competences permis_conduire_b accompagnement_creation_entreprise accompagnement_validation_acquis'  # Needed to preserve the enum order in Python 2
    autre = 'Autre formation'
    bilan_competences = 'Bilan de compétences'
    permis_conduire_b = 'Permis de conduire B (code et/ou conduite)'
    accompagnement_creation_entreprise = "Accompagnement à la création d'entreprise"
    accompagnement_validation_acquis = "Accompagnement à la validation des acquis de l'experience (VAE)"


class dispositifs_formation(Variable):
    value_type = Enum
    possible_values = DispositifsDeFormation
    default_value = DispositifsDeFormation.autre
    entity = Individu
    label = 'Dispositifs de formation'
    definition_period = MONTH


class formation_validee_pole_emploi(Variable):
    value_type = bool
    entity = Individu
    label = "La formation de l'individu est validée par Pôle emploi"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class formation_financee_ou_cofinancee(Variable):
    value_type = bool
    entity = Individu
    label = "La formation de l'individu est financée ou cofinancée (compte personnel de formation (CPF), fonds propres, Pôle Emploi, un tiers)"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class TypesActiviteEnRechercheEmploi(Enum):
    __order__ = 'indeterminee entretien_embauche concours_public examen_certifiant prestation_accompagnement immersion_professionnelle_PMSMP'  # Needed to preserve the enum order in Python 2
    indeterminee = 'Activité indéterminée'
    entretien_embauche = "Entretien d'embauche"
    concours_public = 'Concours public'
    examen_certifiant = 'Examen certifiant'
    prestation_accompagnement = "Prestation d'accompagnement"
    immersion_professionnelle_PMSMP = 'Immersion professionnelle PMSMP'


class types_activite_en_recherche_emploi(Variable):
    value_type = Enum
    possible_values = TypesActiviteEnRechercheEmploi
    default_value = TypesActiviteEnRechercheEmploi.indeterminee
    entity = Individu
    label = "Les types d'activité dans un contexte de recherche d'emploi pour l'aide à la mobilité de Pôle Emploi - AMOB"
    definition_period = MONTH


class aide_mobilite_categories_demandeur_emploi_eligibles(Variable):
    value_type = bool
    entity = Individu
    label = "Le demandeur d'emploi appartient à une catégorie éligible pour l'aide à la mobilité - AMOB"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = 'http://www.bo-pole-emploi.org/bulletinsofficiels/deliberation-n-2021-42-du-8-juin-2021-bope-n2021-43.html?type=dossiers/2021/bope-n-2021-043-du-11-juin-2021'

    def formula_2021_06_09(individu, period, parameters):

        pe_categorie_demandeur_emploi = individu('pole_emploi_categorie_demandeur_emploi', period)

        stagiaire_formation_professionnelle = individu('stagiaire', period)
        contrat_aide = individu('contrat_aide', period)

        categorie_4 = pe_categorie_demandeur_emploi == TypesCategoriesDemandeurEmploi.categorie_4
        categorie_4_stagiaire_formation_professionnelle = categorie_4 * stagiaire_formation_professionnelle

        categorie_5 = pe_categorie_demandeur_emploi == TypesCategoriesDemandeurEmploi.categorie_5
        categorie_5_contrat_aide = categorie_5 * contrat_aide

        categories_eligibles = ((pe_categorie_demandeur_emploi == TypesCategoriesDemandeurEmploi.categorie_1)
                                + (pe_categorie_demandeur_emploi == TypesCategoriesDemandeurEmploi.categorie_2)
                                + (pe_categorie_demandeur_emploi == TypesCategoriesDemandeurEmploi.categorie_3)
                                + (categorie_4_stagiaire_formation_professionnelle + categorie_5_contrat_aide)
                                + (pe_categorie_demandeur_emploi == TypesCategoriesDemandeurEmploi.categorie_6)
                                + (pe_categorie_demandeur_emploi == TypesCategoriesDemandeurEmploi.categorie_7)
                                + (pe_categorie_demandeur_emploi == TypesCategoriesDemandeurEmploi.categorie_8))

        return categories_eligibles


class aide_mobilite_ressources_eligibles(Variable):
    value_type = bool
    entity = Individu
    label = "Le demandeur d'emploi perçoit une indemnité éligible pour l'aide à la mobilité - AMOB"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = 'http://www.bo-pole-emploi.org/bulletinsofficiels/deliberation-n-2021-42-du-8-juin-2021-bope-n2021-43.html?type=dossiers/2021/bope-n-2021-043-du-11-juin-2021'

    def formula_2021_06_09(individu, period, parameters):
        epsilon = 0.0001
        lieu_de_residence = individu.menage('residence', period)
        mayotte = lieu_de_residence == TypesLieuResidence.mayotte
        hors_mayotte = not_(mayotte)

        allocation_individu = individu('allocation_retour_emploi_journaliere', period)
        allocations = parameters(period).chomage.allocation_retour_emploi
        allocation_minimale_hors_mayotte = allocations.montant_minimum_hors_mayotte * hors_mayotte
        allocation_minimale_mayotte = allocations.montant_minimum_mayotte * mayotte

        allocation_minimale_en_fonction_de_la_region = allocation_minimale_hors_mayotte + allocation_minimale_mayotte

        are_individu_egale_are_min = fabs(allocation_individu - allocation_minimale_en_fonction_de_la_region) < epsilon
        are_individu_inferieure_are_min = allocation_individu < allocation_minimale_en_fonction_de_la_region

        return are_individu_inferieure_are_min + are_individu_egale_are_min


class aide_mobilite_eligible(Variable):
    value_type = bool
    entity = Individu
    label = "Eligibilité à l'aide à la mobilité de Pôle Emploi - AMOB"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = 'http://www.bo-pole-emploi.org/bulletinsofficiels/deliberation-n-2021-42-du-8-juin-2021-bope-n2021-43.html?type=dossiers/2021/bope-n-2021-043-du-11-juin-2021'
    documentation = '''
        1- L'individu doit être dans un contexte de recherche d'emploi, reprise d'emploi ou d'entrée en formation
            1.1 - Pour une formation ou une reprise d'emploi, la demande doit être faite au plus tard dans le mois suivant
            1.2 - Pour une recherche d'emploi, la demande doit être faite avant l'entretien d'embauche ou au plus tard dans un délai de 7 jours
        2- Sa reprise d'emploi ou son entretien d'emploi, cela doit concerner un cdi, cdd ou ctt de plus de 3 mois ou une formation de plus de 40h validée et financée
        3- L'individu est inscrit en catégorie 1, 2, 3, 4 "stagiaire de la formation professionnelle" ou 5 "contrat aidé", 6, 7 ou 8
        4- L'individu est non indemnisé ou son allocation est inférieure ou égale à l'ARE minimale
        5- L'emploi ou la formation se situe en France
        6-
            6.1 - Son "activité" doit être à plus de 60 km aller-retour de son lieu de résidence
            6.2 - Ou 20 km lorsque l'individu réside en dehors de la métropole
            6.3 - Ou 2 heures de trajet aller-retour
        7 - Les dispositifs suivants ne donne pas lieu au versement de l'aide à la mobilité :
            - le bilan de compétences
            - le permis de conduire B (code et/ou conduite)
            - l’accompagnement à la création d’entreprise
            - l’accompagnement à la validation des acquis de l’expérience (VAE)
    '''

    def formula_2021_06_09(individu, period, parameters):

        #  1
        contexte = individu('contexte_activite_pole_emploi', period)
        contrat_travail_debut = individu('contrat_de_travail_debut', period)
        date_debut_type_activite_recherche_emploi = individu('date_debut_recherche_emploi', period)
        contrat_de_travail_debut_en_mois = contrat_travail_debut.astype('M8[M]')
        amob_date_de_demande = individu('aide_mobilite_date_demande', period)
        parametres_amob = parameters(period).prestations_sociales.aide_mobilite
        date_contrat_limite_contexte_formation_reprise = min_((contrat_de_travail_debut_en_mois + 1) + (contrat_travail_debut - contrat_de_travail_debut_en_mois),
                                               (contrat_de_travail_debut_en_mois + 2) - timedelta64(1, 'D'))
        dates_demandes_amob_eligibles_formation_reprise = amob_date_de_demande <= date_contrat_limite_contexte_formation_reprise

        date_limite_contrat_contexte_recherche = date_debut_type_activite_recherche_emploi + (parametres_amob.delai_max - 1)  # 7 jours de date à date
        dates_demandes_amob_eligibles_recherche = amob_date_de_demande <= date_limite_contrat_contexte_recherche

        en_recherche_emploi = contexte == ContexteActivitePoleEmploi.recherche_emploi
        en_reprise_emploi = contexte == ContexteActivitePoleEmploi.reprise_emploi
        en_formation = contexte == ContexteActivitePoleEmploi.formation

        contextes_eligibles = (en_recherche_emploi * dates_demandes_amob_eligibles_recherche)\
            + ((en_reprise_emploi + en_formation) * dates_demandes_amob_eligibles_formation_reprise)

        #  2
        activite_en_recherche_emploi = individu('types_activite_en_recherche_emploi', period)
        reprises_emploi_types_activites = individu('contrat_de_travail_type', period)
        formation_validee = individu('formation_validee_pole_emploi', period)
        formation_financee = individu('formation_financee_ou_cofinancee', period)

        # Activites en recherche d'emploi
        en_entretien_embauche = (activite_en_recherche_emploi == TypesActiviteEnRechercheEmploi.entretien_embauche) * en_recherche_emploi

        activites_en_recherche_emploi_eligibles = not_((activite_en_recherche_emploi == TypesActiviteEnRechercheEmploi.entretien_embauche)
            + (activite_en_recherche_emploi == TypesActiviteEnRechercheEmploi.indeterminee))\
            * en_recherche_emploi

        reprises_types_activites_formation = reprises_emploi_types_activites == TypesContrat.formation
        reprises_types_activites_cdi = reprises_emploi_types_activites == TypesContrat.cdi
        reprises_types_activites_cdd_ctt = (reprises_emploi_types_activites == TypesContrat.cdd) + (reprises_emploi_types_activites == TypesContrat.ctt)

        #  La formation doit être supérieure ou égale à 40 heures
        duree_formation = individu('duree_formation', period)
        periode_formation_eligible = duree_formation >= parametres_amob.duree_de_formation_minimum

        #  La durée de contrat de l'emploi doit être d'au moins 3 mois
        duree_de_contrat_3_mois_minimum = individu('contrat_de_travail_duree', period) >= parametres_amob.duree_de_contrat_minimum

        reprises_cdd_ctt_eligibles = reprises_types_activites_cdd_ctt * duree_de_contrat_3_mois_minimum

        types_et_duree_activite_eligibles = (((reprises_types_activites_cdi + reprises_cdd_ctt_eligibles) * (en_reprise_emploi + en_entretien_embauche))
                                            + (reprises_types_activites_formation
                                               * en_formation
                                               * formation_validee
                                               * formation_financee
                                               * periode_formation_eligible))

        activites_eligibles = (types_et_duree_activite_eligibles + activites_en_recherche_emploi_eligibles)

        #  3
        categories_eligibles = individu('aide_mobilite_categories_demandeur_emploi_eligibles', period)

        #  4
        montants_allocations_eligibles = individu('aide_mobilite_ressources_eligibles', period)

        #  5
        lieux_activite_eligibles = not_(individu('lieu_emploi_ou_formation', period) == TypesLieuEmploiFormation.non_renseigne)

        #  6
        lieu_de_residence = individu.menage('residence', period)
        temps_de_trajet = individu('aide_mobilite_duree_trajet', period)
        distance_aller_retour = individu('distance_activite_domicile', period) * 2

        temps_de_trajet_min = parametres_amob.duree_trajet_minimum
        reside_en_metropole = lieu_de_residence == TypesLieuResidence.metropole
        residence_renseignee = not_(lieu_de_residence == TypesLieuResidence.non_renseigne)

        distances_et_durees_aller_retour_eligibles = (((distance_aller_retour > parametres_amob.distance_minimum.metropole) * reside_en_metropole)
                                                    + ((distance_aller_retour > parametres_amob.distance_minimum.hors_metropole) * (not_(reside_en_metropole) * residence_renseignee))
                                                    + ((temps_de_trajet > temps_de_trajet_min) * residence_renseignee))

        #  7
        dispositifs_formations = individu('dispositifs_formation', period)
        dispositifs_formations_eligibles = dispositifs_formations == DispositifsDeFormation.autre

        eligibilite_amob = (contextes_eligibles
                       * activites_eligibles
                       * categories_eligibles
                       * montants_allocations_eligibles
                       * lieux_activite_eligibles
                       * distances_et_durees_aller_retour_eligibles
                       * dispositifs_formations_eligibles)

        return eligibilite_amob


class aide_mobilite(Variable):
    value_type = float
    entity = Individu
    label = "Montant de l'aide à la mobilité de Pôle emploi - AMOB"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = 'http://www.bo-pole-emploi.org/bulletinsofficiels/deliberation-n-2021-42-du-8-juin-2021-bope-n2021-43.html?type=dossiers/2021/bope-n-2021-043-du-11-juin-2021'

    def formula_2021_06_09(individu, period, parameters):

        eligibilite_amob = individu('aide_mobilite_eligible', period)
        parametres_amob = parameters(period).prestations_sociales.aide_mobilite

        annee_glissante = Period(('year', period.start, 1)).offset(-1)

        aide_mobilite_12_derniers_mois = individu('aide_mobilite', annee_glissante, options=[ADD])

        montant_max = parametres_amob.montants.maximum
        montant_amob_deja_percu = min_(montant_max, fabs(aide_mobilite_12_derniers_mois))
        distance_aller_retour = individu('distance_activite_domicile', period)
        nb_aller_retour = individu('nombre_allers_retours', period)
        nb_nuitees = individu('nuitees', period)
        nb_repas = individu('repas', period)

        montant = parametres_amob.montants

        montants_frais_deplacement = montant.deplacement * distance_aller_retour * nb_aller_retour
        montants_frais_hebergement = montant.hebergement * nb_nuitees
        montants_frais_repas = montant.repas * nb_repas

        montants_theoriques = montants_frais_deplacement + montants_frais_hebergement + montants_frais_repas
        montants_max_attribuables = montant_max - montant_amob_deja_percu

        montants_reels = min_(montants_theoriques, montants_max_attribuables)

        return montants_reels * eligibilite_amob


class aide_mobilite_bon_de_transport(Variable):
    value_type = bool
    entity = Individu
    label = "Eligibilité à un bon de transport, part de l'aide à la mobilité de Pôle emploi (AMOB)"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = 'http://www.bo-pole-emploi.org/bulletinsofficiels/deliberation-n-2021-42-du-8-juin-2021-bope-n2021-43.html?type=dossiers/2021/bope-n-2021-043-du-11-juin-2021'

    def formula_2021_06_09(individu, period):

        contexte = individu('contexte_activite_pole_emploi', period)
        en_recherche_emploi = contexte == ContexteActivitePoleEmploi.recherche_emploi
        lieux_activite_eligibles = individu('lieu_emploi_ou_formation', period) == TypesLieuEmploiFormation.metropole_hors_corse
        categories_non_eligibles = not_(individu('aide_mobilite_categories_demandeur_emploi_eligibles', period))
        allocation_non_eligible = not_(individu('aide_mobilite_ressources_eligibles', period))

        return (en_recherche_emploi
            * lieux_activite_eligibles
            * categories_non_eligibles
            * allocation_non_eligible)



import logging
from openfisca_france.model.base import *

log = logging.getLogger(__name__)


class csg(Variable):
    value_type = float
    entity = Individu
    label = 'Contribution sociale généralisée'
    definition_period = YEAR

    def formula(individu, period):
        csg_imposable_salaire = individu('csg_imposable_salaire', period, options = [ADD])
        csg_deductible_salaire = individu('csg_deductible_salaire', period, options = [ADD])
        csg_imposable_chomage = individu('csg_imposable_chomage', period, options = [ADD])
        csg_deductible_chomage = individu('csg_deductible_chomage', period, options = [ADD])
        csg_imposable_retraite = individu('csg_imposable_retraite', period, options = [ADD])
        csg_deductible_retraite = individu('csg_deductible_retraite', period, options = [ADD])
        csg_imposable_non_salarie = individu('csg_imposable_non_salarie', period, options = [ADD])
        csg_deductible_non_salarie = individu('csg_deductible_non_salarie', period, options = [ADD])
        # CSG sur revenus du capital, définie à l'échelle du foyer fiscal, mais projetée sur le déclarant principal
        csg_revenus_capital = individu.foyer_fiscal('csg_revenus_capital', period)
        csg_revenus_capital_projetee = csg_revenus_capital * individu.has_role(FoyerFiscal.DECLARANT_PRINCIPAL)

        return (
            csg_imposable_salaire
            + csg_deductible_salaire
            + csg_imposable_chomage
            + csg_deductible_chomage
            + csg_imposable_retraite
            + csg_deductible_retraite
            + csg_imposable_non_salarie
            + csg_deductible_non_salarie
            + csg_revenus_capital_projetee
            )

    # TODO: manque CSG sur IJ et pré-retraites


class crds(Variable):
    value_type = float
    entity = Individu
    label = 'Contributions au remboursement de la dette sociale'
    reference = [
        'Ordonnance n° 96-50 du 24 janvier 1996 relative au remboursement de la dette sociale, art. 14',
        'https://www.legifrance.gouv.fr/loda/id/LEGISCTA000006106400'
        ]
    definition_period = YEAR

    def formula(individu, period):
        # CRDS sur revenus individuels
        crds_salaire = individu('crds_salaire', period, options = [ADD])
        crds_retraite = individu('crds_retraite', period, options = [ADD])
        crds_chomage = individu('crds_chomage', period, options = [ADD])
        crds_non_salarie = individu('crds_non_salarie', period, options = [ADD])
        crds_individu = crds_salaire + crds_retraite + crds_chomage + crds_non_salarie
        # CRDS sur revenus de la famille, projetés seulement sur la première personne
        crds_pfam = individu.famille('crds_pfam', period)
        crds_logement = individu.famille('crds_logement', period, options = [ADD])
        crds_mini = individu.famille('crds_mini', period, options = [ADD])
        crds_famille = crds_pfam + crds_logement + crds_mini
        crds_famille_projetes = crds_famille * individu.has_role(Famille.DEMANDEUR)
        # CRDS sur revenus du capital, définie à l'échelle du foyer fiscal, mais projetée sur le déclarant principal
        crds_revenus_capital = individu.foyer_fiscal('crds_revenus_capital', period)
        crds_revenus_capital_projetee = crds_revenus_capital * individu.has_role(FoyerFiscal.DECLARANT_PRINCIPAL)

        return crds_individu + crds_famille_projetes + crds_revenus_capital_projetee


class crds_hors_prestations(Variable):
    value_type = float
    entity = Individu
    label = 'Contributions au remboursement de la dette sociale (hors celles portant sur les prestations sociales)'
    definition_period = YEAR

    def formula(individu, period):
        # CRDS sur revenus individuels
        crds_salaire = individu('crds_salaire', period, options = [ADD])
        crds_retraite = individu('crds_retraite', period, options = [ADD])
        crds_chomage = individu('crds_chomage', period, options = [ADD])
        crds_non_salarie = individu('crds_non_salarie', period, options = [ADD])
        crds_individu = crds_salaire + crds_retraite + crds_chomage + crds_non_salarie
        # CRDS sur revenus du capital, définie à l'échelle du foyer fiscal, mais projetée sur le déclarant principal
        crds_revenus_capital = individu.foyer_fiscal('crds_revenus_capital', period)
        crds_revenus_capital_projetee = crds_revenus_capital * individu.has_role(FoyerFiscal.DECLARANT_PRINCIPAL)

        return crds_individu + crds_revenus_capital_projetee



from numpy import logical_not as not_

from openfisca_core.periods import Period

from openfisca_france.model.base import Variable, Famille, MONTH, ADD, set_input_dispatch_by_period


class css_cmu_acs_eligibilite(Variable):
    value_type = bool
    entity = Famille
    label = "Pré-éligibilité à l'ACS, la CMU-C et à la CSS, avant prise en compte des ressources"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(famille, period, parameters):
        previous_year = Period(('year', period.start, 1)).offset(-1)
        this_year = period.this_year
        age_min = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.cs.cmu.age_limite_pac
        nb_enfants = famille('cmu_nb_pac', period)

        # Une personne de 25 ans ne doit pas être à charge fiscale, ni hébergée par ses parents, ni recevoir de pensions alimentaires pour pouvoir bénéficier de la CMU individuellement.
        a_charge_des_parents = famille.members('enfant_a_charge', this_year)
        habite_chez_parents = famille.members('habite_chez_parents', period)
        recoit_pension = famille.members('pensions_alimentaires_percues', previous_year, options = [ADD]) > 0
        condition_independance = not_(a_charge_des_parents + habite_chez_parents + recoit_pension)

        age = famille.members('age', period)
        condition_age = (age >= age_min)

        eligibilite_famille = (
            (nb_enfants > 0)
            + famille.any(condition_age)
            + famille.all(condition_independance)
            )

        return eligibilite_famille
